@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * nsBaseContentList is a basic list of content nodes; nsContentList
@@ -44,36 +12,22 @@
 #ifndef nsContentList_h___
 #define nsContentList_h___
 
+#include "mozilla/Attributes.h"
+#include "nsContentListDeclarations.h"
 #include "nsISupports.h"
-#include "nsCOMArray.h"
+#include "nsTArray.h"
 #include "nsString.h"
 #include "nsIHTMLCollection.h"
 #include "nsIDOMNodeList.h"
 #include "nsINodeList.h"
 #include "nsStubMutationObserver.h"
 #include "nsIAtom.h"
-#include "nsINameSpaceManager.h"
 #include "nsCycleCollectionParticipant.h"
+#include "nsNameSpaceManager.h"
 #include "nsWrapperCache.h"
-#include "nsCRT.h"
 #include "nsHashKeys.h"
+#include "mozilla/HashFunctions.h"
 
-// Magic namespace id that means "match all namespaces".  This is
-// negative so it won't collide with actual namespace constants.
-#define kNameSpaceID_Wildcard PR_INT32_MIN
-
-// This is a callback function type that can be used to implement an
-// arbitrary matching algorithm.  aContent is the content that may
-// match the list, while aNamespaceID, aAtom, and aData are whatever
-// was passed to the list's constructor.
-typedef PRBool (*nsContentListMatchFunc)(nsIContent* aContent,
-                                         PRInt32 aNamespaceID,
-                                         nsIAtom* aAtom,
-                                         void* aData);
-
-typedef void (*nsContentListDestroyFunc)(void* aData);
-
-class nsIDocument;
 namespace mozilla {
 namespace dom {
 class Element;
@@ -84,6 +38,10 @@ class Element;
 class nsBaseContentList : public nsINodeList
 {
 public:
+  nsBaseContentList()
+  {
+    SetIsDOMBinding();
+  }
   virtual ~nsBaseContentList();
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
@@ -92,16 +50,19 @@ public:
   NS_DECL_NSIDOMNODELIST
 
   // nsINodeList
-  virtual nsIContent* GetNodeAt(PRUint32 aIndex);
-  virtual PRInt32 IndexOf(nsIContent* aContent);
-  
-  PRUint32 Length() const { 
-    return mElements.Count();
+  virtual int32_t IndexOf(nsIContent* aContent) MOZ_OVERRIDE;
+  virtual nsIContent* Item(uint32_t aIndex) MOZ_OVERRIDE;
+
+  uint32_t Length() const { 
+    return mElements.Length();
   }
 
-  NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsBaseContentList, nsINodeList)
+  NS_DECL_CYCLE_COLLECTION_SKIPPABLE_SCRIPT_HOLDER_CLASS(nsBaseContentList)
 
-  void AppendElement(nsIContent *aContent);
+  void AppendElement(nsIContent *aContent)
+  {
+    mElements.AppendElement(aContent);
+  }
   void MaybeAppendElement(nsIContent* aContent)
   {
     if (aContent)
@@ -114,19 +75,40 @@ public:
    * @param aContent Element to insert, must not be null
    * @param aIndex Index to insert the element at.
    */
-  void InsertElementAt(nsIContent* aContent, PRInt32 aIndex);
+  void InsertElementAt(nsIContent* aContent, int32_t aIndex)
+  {
+    NS_ASSERTION(aContent, "Element to insert must not be null");
+    mElements.InsertElementAt(aIndex, aContent);
+  }
 
-  void RemoveElement(nsIContent *aContent); 
+  void RemoveElement(nsIContent *aContent)
+  {
+    mElements.RemoveElement(aContent);
+  }
 
   void Reset() {
     mElements.Clear();
   }
 
+  virtual int32_t IndexOf(nsIContent *aContent, bool aDoFlush);
 
-  virtual PRInt32 IndexOf(nsIContent *aContent, PRBool aDoFlush);
+  virtual JSObject* WrapObject(JSContext *cx)
+    MOZ_OVERRIDE = 0;
 
+  void SetCapacity(uint32_t aCapacity)
+  {
+    mElements.SetCapacity(aCapacity);
+  }
 protected:
-  nsCOMArray<nsIContent> mElements;
+  /**
+   * To be called from non-destructor locations (e.g. unlink) that want to
+   * remove from caches.  Cacheable subclasses should override.
+   */
+  virtual void RemoveFromCaches()
+  {
+  }
+
+  nsTArray< nsCOMPtr<nsIContent> > mElements;
 };
 
 
@@ -142,26 +124,17 @@ public:
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsSimpleContentList,
                                            nsBaseContentList)
 
-  virtual nsINode* GetParentObject()
+  virtual nsINode* GetParentObject() MOZ_OVERRIDE
   {
     return mRoot;
   }
+  virtual JSObject* WrapObject(JSContext *cx) MOZ_OVERRIDE;
 
 private:
   // This has to be a strong reference, the root might go away before the list.
   nsCOMPtr<nsINode> mRoot;
 };
 
-// This class is used only by form element code and this is a static
-// list of elements. NOTE! This list holds strong references to
-// the elements in the list.
-class nsFormContentList : public nsSimpleContentList
-{
-public:
-  nsFormContentList(nsIContent *aForm,
-                    nsBaseContentList& aContentList);
-};
-
 /**
  * Class that's used as the key to hash nsContentList implementations
  * for fast retrieval
@@ -169,32 +142,33 @@ public:
 struct nsContentListKey
 {
   nsContentListKey(nsINode* aRootNode,
-                   PRInt32 aMatchNameSpaceId,
+                   int32_t aMatchNameSpaceId,
                    const nsAString& aTagname)
     : mRootNode(aRootNode),
       mMatchNameSpaceId(aMatchNameSpaceId),
-      mTagname(aTagname)
+      mTagname(aTagname),
+      mHash(mozilla::AddToHash(mozilla::HashString(aTagname), mRootNode,
+                               mMatchNameSpaceId))
   {
   }
 
   nsContentListKey(const nsContentListKey& aContentListKey)
     : mRootNode(aContentListKey.mRootNode),
       mMatchNameSpaceId(aContentListKey.mMatchNameSpaceId),
-      mTagname(aContentListKey.mTagname)
+      mTagname(aContentListKey.mTagname),
+      mHash(aContentListKey.mHash)
   {
   }
 
-  inline PRUint32 GetHash(void) const
+  inline uint32_t GetHash(void) const
   {
-    return
-      HashString(mTagname) ^
-      (NS_PTR_TO_INT32(mRootNode) << 12) ^
-      (mMatchNameSpaceId << 24);
+    return mHash;
   }
   
   nsINode* const mRootNode; // Weak ref
-  const PRInt32 mMatchNameSpaceId;
+  const int32_t mMatchNameSpaceId;
   const nsAString& mTagname;
+  const uint32_t mHash;
 };
 
 /**
@@ -245,10 +219,10 @@ public:
    *              our root.
    */  
   nsContentList(nsINode* aRootNode,
-                PRInt32 aMatchNameSpaceId,
+                int32_t aMatchNameSpaceId,
                 nsIAtom* aHTMLMatchAtom,
                 nsIAtom* aXMLMatchAtom,
-                PRBool aDeep = PR_TRUE);
+                bool aDeep = true);
 
   /**
    * @param aRootNode The node under which to limit our search.
@@ -270,33 +244,50 @@ public:
                 nsContentListMatchFunc aFunc,
                 nsContentListDestroyFunc aDestroyFunc,
                 void* aData,
-                PRBool aDeep = PR_TRUE,
-                nsIAtom* aMatchAtom = nsnull,
-                PRInt32 aMatchNameSpaceId = kNameSpaceID_None,
-                PRBool aFuncMayDependOnAttr = PR_TRUE);
+                bool aDeep = true,
+                nsIAtom* aMatchAtom = nullptr,
+                int32_t aMatchNameSpaceId = kNameSpaceID_None,
+                bool aFuncMayDependOnAttr = true);
   virtual ~nsContentList();
 
+  // nsWrapperCache
+  using nsWrapperCache::GetWrapperPreserveColor;
+  virtual JSObject* WrapObject(JSContext* aCx) MOZ_OVERRIDE;
+protected:
+  virtual JSObject* GetWrapperPreserveColorInternal() MOZ_OVERRIDE
+  {
+    return nsWrapperCache::GetWrapperPreserveColor();
+  }
+public:
+
   // nsIDOMHTMLCollection
   NS_DECL_NSIDOMHTMLCOLLECTION
 
   // nsBaseContentList overrides
-  virtual PRInt32 IndexOf(nsIContent *aContent, PRBool aDoFlush);
-  virtual nsIContent* GetNodeAt(PRUint32 aIndex);
-  virtual PRInt32 IndexOf(nsIContent* aContent);
-  virtual nsINode* GetParentObject()
+  virtual int32_t IndexOf(nsIContent *aContent, bool aDoFlush) MOZ_OVERRIDE;
+  virtual int32_t IndexOf(nsIContent* aContent) MOZ_OVERRIDE;
+  virtual nsINode* GetParentObject() MOZ_OVERRIDE
   {
     return mRootNode;
   }
 
-  // nsIHTMLCollection
-  // GetNodeAt already declared as part of nsINodeList
-  virtual nsISupports* GetNamedItem(const nsAString& aName,
-                                    nsWrapperCache** aCache);
+  virtual nsIContent* Item(uint32_t aIndex) MOZ_OVERRIDE;
+  virtual mozilla::dom::Element* GetElementAt(uint32_t index) MOZ_OVERRIDE;
+  virtual mozilla::dom::Element*
+  GetFirstNamedElement(const nsAString& aName, bool& aFound) MOZ_OVERRIDE
+  {
+    mozilla::dom::Element* item = NamedItem(aName, true);
+    aFound = !!item;
+    return item;
+  }
+  virtual void GetSupportedNames(unsigned aFlags,
+                                 nsTArray<nsString>& aNames) MOZ_OVERRIDE;
 
   // nsContentList public methods
-  NS_HIDDEN_(PRUint32) Length(PRBool aDoFlush);
-  NS_HIDDEN_(nsIContent*) Item(PRUint32 aIndex, PRBool aDoFlush);
-  NS_HIDDEN_(nsIContent*) NamedItem(const nsAString& aName, PRBool aDoFlush);
+  NS_HIDDEN_(uint32_t) Length(bool aDoFlush);
+  NS_HIDDEN_(nsIContent*) Item(uint32_t aIndex, bool aDoFlush);
+  NS_HIDDEN_(mozilla::dom::Element*)
+  NamedItem(const nsAString& aName, bool aDoFlush);
 
   // nsIMutationObserver
   NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTECHANGED
@@ -321,7 +312,7 @@ public:
     return static_cast<nsContentList*>(list);
   }
 
-  PRBool MatchesKey(const nsContentListKey& aKey) const
+  bool MatchesKey(const nsContentListKey& aKey) const
   {
     // The root node is most commonly the same: the document.  And the
     // most common namespace id is kNameSpaceID_Unknown.  So check the
@@ -334,6 +325,16 @@ public:
       mMatchNameSpaceId == aKey.mMatchNameSpaceId;
   }
 
+  /**
+   * Sets the state to LIST_DIRTY and clears mElements array.
+   * @note This is the only acceptable way to set state to LIST_DIRTY.
+   */
+  void SetDirty()
+  {
+    mState = LIST_DIRTY;
+    Reset();
+  }
+
 protected:
   /**
    * Returns whether the element matches our criterion
@@ -341,7 +342,7 @@ protected:
    * @param  aElement the element to attempt to match
    * @return whether we match
    */
-  PRBool Match(mozilla::dom::Element *aElement);
+  bool Match(mozilla::dom::Element *aElement);
   /**
    * See if anything in the subtree rooted at aContent, including
    * aContent itself, matches our criterion.
@@ -349,7 +350,7 @@ protected:
    * @param  aContent the root of the subtree to match against
    * @return whether we match something in the tree rooted at aContent
    */
-  PRBool MatchSelf(nsIContent *aContent);
+  bool MatchSelf(nsIContent *aContent);
 
   /**
    * Populate our list.  Stop once we have at least aNeededLength
@@ -360,16 +361,16 @@ protected:
    * @param aNeededLength the length the list should have when we are
    *        done (unless it exhausts the document)   
    */
-  void PopulateSelf(PRUint32 aNeededLength);
+  void PopulateSelf(uint32_t aNeededLength);
 
   /**
    * @param  aContainer a content node which must be a descendant of
    *         mRootNode
-   * @return PR_TRUE if children or descendants of aContainer could match our
+   * @return true if children or descendants of aContainer could match our
    *                 criterion.
-   *         PR_FALSE otherwise.
+   *         false otherwise.
    */
-  PRBool MayContainRelevantNodes(nsINode* aContainer)
+  bool MayContainRelevantNodes(nsINode* aContainer)
   {
     return mDeep || aContainer == mRootNode;
   }
@@ -383,29 +384,20 @@ protected:
    * If state is not LIST_UP_TO_DATE, fully populate ourselves with
    * all the nodes we can find.
    */
-  inline void BringSelfUpToDate(PRBool aDoFlush);
-
-  /**
-   * Sets the state to LIST_DIRTY and clears mElements array.
-   * @note This is the only acceptable way to set state to LIST_DIRTY.
-   */
-  void SetDirty()
-  {
-    mState = LIST_DIRTY;
-    Reset();
-  }
+  inline void BringSelfUpToDate(bool aDoFlush);
 
   /**
    * To be called from non-destructor locations that want to remove from caches.
    * Needed because if subclasses want to have cache behavior they can't just
    * override RemoveFromHashtable(), since we call that in our destructor.
    */
-  virtual void RemoveFromCaches() {
+  virtual void RemoveFromCaches() MOZ_OVERRIDE
+  {
     RemoveFromHashtable();
   }
 
   nsINode* mRootNode; // Weak ref
-  PRInt32 mMatchNameSpaceId;
+  int32_t mMatchNameSpaceId;
   nsCOMPtr<nsIAtom> mHTMLMatchAtom;
   nsCOMPtr<nsIAtom> mXMLMatchAtom;
 
@@ -426,31 +418,31 @@ protected:
    * The current state of the list (possible values are:
    * LIST_UP_TO_DATE, LIST_LAZY, LIST_DIRTY
    */
-  PRUint8 mState;
+  uint8_t mState;
 
-  // The booleans have to use PRUint8 to pack with mState, because MSVC won't
+  // The booleans have to use uint8_t to pack with mState, because MSVC won't
   // pack different typedefs together.  Once we no longer have to worry about
-  // flushes in XML documents, we can go back to using PRPackedBool for the
+  // flushes in XML documents, we can go back to using bool for the
   // booleans.
   
   /**
    * True if we are looking for elements named "*"
    */
-  PRUint8 mMatchAll : 1;
+  uint8_t mMatchAll : 1;
   /**
    * Whether to actually descend the tree.  If this is false, we won't
    * consider grandkids of mRootNode.
    */
-  PRUint8 mDeep : 1;
+  uint8_t mDeep : 1;
   /**
    * Whether the return value of mFunc could depend on the values of
    * attributes.
    */
-  PRUint8 mFuncMayDependOnAttr : 1;
+  uint8_t mFuncMayDependOnAttr : 1;
   /**
    * Whether we actually need to flush to get our state correct.
    */
-  PRUint8 mFlushesNeeded : 1;
+  uint8_t mFlushesNeeded : 1;
 
 #ifdef DEBUG_CONTENT_LIST
   void AssertInSync();
@@ -462,7 +454,7 @@ protected:
  */
 class nsCacheableFuncStringContentList;
 
-class NS_STACK_CLASS nsFuncStringCacheKey {
+class MOZ_STACK_CLASS nsFuncStringCacheKey {
 public:
   nsFuncStringCacheKey(nsINode* aRootNode,
                        nsContentListMatchFunc aFunc,
@@ -472,10 +464,10 @@ public:
     mString(aString)
     {}
 
-  PRUint32 GetHash(void) const
+  uint32_t GetHash(void) const
   {
-    return NS_PTR_TO_INT32(mRootNode) ^ (NS_PTR_TO_INT32(mFunc) << 12) ^
-      nsCRT::HashCode(mString.BeginReading(), mString.Length());
+    uint32_t hash = mozilla::HashString(mString);
+    return mozilla::AddToHash(hash, mRootNode, mFunc);
   }
 
 private:
@@ -486,38 +478,39 @@ private:
   const nsAString& mString;
 };
 
-/**
- * A function that allocates the matching data for this
- * FuncStringContentList.  Returning aString is perfectly fine; in
- * that case the destructor function should be a no-op.
- */
-typedef void* (*nsFuncStringContentListDataAllocator)(nsINode* aRootNode,
-                                                      const nsString* aString);
-
 // aDestroyFunc is allowed to be null
+// aDataAllocator must always return a non-null pointer
 class nsCacheableFuncStringContentList : public nsContentList {
 public:
+  virtual ~nsCacheableFuncStringContentList();
+
+  bool Equals(const nsFuncStringCacheKey* aKey) {
+    return mRootNode == aKey->mRootNode && mFunc == aKey->mFunc &&
+      mString == aKey->mString;
+  }
+
+#ifdef DEBUG
+  enum ContentListType {
+    eNodeList,
+    eHTMLCollection
+  };
+  ContentListType mType;
+#endif
+
+protected:
   nsCacheableFuncStringContentList(nsINode* aRootNode,
                                    nsContentListMatchFunc aFunc,
                                    nsContentListDestroyFunc aDestroyFunc,
                                    nsFuncStringContentListDataAllocator aDataAllocator,
                                    const nsAString& aString) :
-    nsContentList(aRootNode, aFunc, aDestroyFunc, nsnull),
+    nsContentList(aRootNode, aFunc, aDestroyFunc, nullptr),
     mString(aString)
   {
     mData = (*aDataAllocator)(aRootNode, &mString);
+    MOZ_ASSERT(mData);
   }
 
-  virtual ~nsCacheableFuncStringContentList();
-
-  PRBool Equals(const nsFuncStringCacheKey* aKey) {
-    return mRootNode == aKey->mRootNode && mFunc == aKey->mFunc &&
-      mString == aKey->mString;
-  }
-
-  PRBool AllocatedData() const { return !!mData; }
-protected:
-  virtual void RemoveFromCaches() {
+  virtual void RemoveFromCaches() MOZ_OVERRIDE {
     RemoveFromFuncStringHashtable();
   }
   void RemoveFromFuncStringHashtable();
@@ -525,20 +518,52 @@ protected:
   nsString mString;
 };
 
-// If aMatchNameSpaceId is kNameSpaceID_Unknown, this will return a
-// content list which matches ASCIIToLower(aTagname) against HTML
-// elements in HTML documents and aTagname against everything else.
-// For any other value of aMatchNameSpaceId, the list will match
-// aTagname against all elements.
-already_AddRefed<nsContentList>
-NS_GetContentList(nsINode* aRootNode,
-                  PRInt32 aMatchNameSpaceId,
-                  const nsAString& aTagname);
+class nsCacheableFuncStringNodeList
+  : public nsCacheableFuncStringContentList
+{
+public:
+  nsCacheableFuncStringNodeList(nsINode* aRootNode,
+                                nsContentListMatchFunc aFunc,
+                                nsContentListDestroyFunc aDestroyFunc,
+                                nsFuncStringContentListDataAllocator aDataAllocator,
+                                const nsAString& aString)
+    : nsCacheableFuncStringContentList(aRootNode, aFunc, aDestroyFunc,
+                                       aDataAllocator, aString)
+  {
+#ifdef DEBUG
+    mType = eNodeList;
+#endif
+  }
+
+  virtual JSObject* WrapObject(JSContext *cx) MOZ_OVERRIDE;
+
+#ifdef DEBUG
+  static const ContentListType sType;
+#endif
+};
+
+class nsCacheableFuncStringHTMLCollection
+  : public nsCacheableFuncStringContentList
+{
+public:
+  nsCacheableFuncStringHTMLCollection(nsINode* aRootNode,
+                                      nsContentListMatchFunc aFunc,
+                                      nsContentListDestroyFunc aDestroyFunc,
+                                      nsFuncStringContentListDataAllocator aDataAllocator,
+                                      const nsAString& aString)
+    : nsCacheableFuncStringContentList(aRootNode, aFunc, aDestroyFunc,
+                                       aDataAllocator, aString)
+  {
+#ifdef DEBUG
+    mType = eHTMLCollection;
+#endif
+  }
+
+  virtual JSObject* WrapObject(JSContext *cx) MOZ_OVERRIDE;
+
+#ifdef DEBUG
+  static const ContentListType sType;
+#endif
+};
 
-already_AddRefed<nsContentList>
-NS_GetFuncStringContentList(nsINode* aRootNode,
-                            nsContentListMatchFunc aFunc,
-                            nsContentListDestroyFunc aDestroyFunc,
-                            nsFuncStringContentListDataAllocator aDataAllocator,
-                            const nsAString& aString);
 #endif // nsContentList_h___
