@@ -1,52 +1,22 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * A class which represents a fragment of text (eg inside a text
  * node); if only codepoints below 256 are used, the text is stored as
- * a char*; otherwise the text is stored as a PRUnichar*
+ * a char*; otherwise the text is stored as a char16_t*
  */
 
 #ifndef nsTextFragment_h___
 #define nsTextFragment_h___
 
+#include "mozilla/Attributes.h"
+#include "mozilla/MemoryReporting.h"
+
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsTraceRefcnt.h"
-#include "nsDOMMemoryReporter.h"
+#include "nsISupportsImpl.h"
 
 class nsString;
 class nsCString;
@@ -55,22 +25,6 @@ class nsCString;
 
 // XXX nsTextFragmentPool?
 
-// XXX these need I18N spankage
-#define XP_IS_SPACE(_ch) \
-  (((_ch) == ' ') || ((_ch) == '\t') || ((_ch) == '\n') || ((_ch) == '\r'))
-
-#define XP_IS_UPPERCASE(_ch) \
-  (((_ch) >= 'A') && ((_ch) <= 'Z'))
-
-#define XP_IS_LOWERCASE(_ch) \
-  (((_ch) >= 'a') && ((_ch) <= 'z'))
-
-#define XP_TO_LOWER(_ch) ((_ch) | 32)
-
-#define XP_TO_UPPER(_ch) ((_ch) & ~32)
-
-#define XP_IS_SPACE_W XP_IS_SPACE
-
 /**
  * A fragment of text. If mIs2b is 1 then the m2b pointer is valid
  * otherwise the m1b pointer is valid. If m1b is used then each byte
@@ -80,7 +34,7 @@ class nsCString;
  * This class does not have a virtual destructor therefore it is not
  * meant to be subclassed.
  */
-class NS_FINAL_CLASS nsTextFragment {
+class nsTextFragment MOZ_FINAL {
 public:
   static nsresult Init();
   static void Shutdown();
@@ -89,7 +43,7 @@ public:
    * Default constructor. Initialize the fragment to be empty.
    */
   nsTextFragment()
-    : m1b(nsnull), mAllBits(0)
+    : m1b(nullptr), mAllBits(0)
   {
     MOZ_COUNT_CTOR(nsTextFragment);
     NS_ASSERTION(sizeof(FragmentBits) == 4, "Bad field packing!");
@@ -99,12 +53,12 @@ public:
 
   /**
    * Change the contents of this fragment to be a copy of the
-   * the argument fragment.
+   * the argument fragment, or to "" if unable to allocate enough memory.
    */
   nsTextFragment& operator=(const nsTextFragment& aOther);
 
   /**
-   * Return true if this fragment is represented by PRUnichar data
+   * Return true if this fragment is represented by char16_t data
    */
   bool Is2b() const
   {
@@ -122,9 +76,9 @@ public:
   }
 
   /**
-   * Get a pointer to constant PRUnichar data.
+   * Get a pointer to constant char16_t data.
    */
-  const PRUnichar *Get2b() const
+  const char16_t *Get2b() const
   {
     NS_ASSERTION(Is2b(), "not 2b text"); 
     return m2b;
@@ -143,7 +97,7 @@ public:
    * Get the length of the fragment. The length is the number of logical
    * characters, not the number of bytes to store the characters.
    */
-  PRUint32 GetLength() const
+  uint32_t GetLength() const
   {
     return mState.mLength;
   }
@@ -158,24 +112,40 @@ public:
    * buffer. If aUpdateBidi is true, contents of the fragment will be scanned,
    * and mState.mIsBidi will be turned on if it includes any Bidi characters.
    */
-  void SetTo(const PRUnichar* aBuffer, PRInt32 aLength, bool aUpdateBidi);
+  bool SetTo(const char16_t* aBuffer, int32_t aLength, bool aUpdateBidi);
 
   /**
    * Append aData to the end of this fragment. If aUpdateBidi is true, contents
    * of the fragment will be scanned, and mState.mIsBidi will be turned on if
    * it includes any Bidi characters.
    */
-  void Append(const PRUnichar* aBuffer, PRUint32 aLength, bool aUpdateBidi);
+  bool Append(const char16_t* aBuffer, uint32_t aLength, bool aUpdateBidi);
 
   /**
    * Append the contents of this string fragment to aString
    */
   void AppendTo(nsAString& aString) const {
+    if (!AppendTo(aString, mozilla::fallible_t())) {
+      NS_ABORT_OOM(GetLength());
+    }
+  }
+
+  /**
+   * Append the contents of this string fragment to aString
+   * @return false if an out of memory condition is detected, true otherwise
+   */
+  bool AppendTo(nsAString& aString,
+                const mozilla::fallible_t&) const NS_WARN_UNUSED_RESULT {
     if (mState.mIs2b) {
-      aString.Append(m2b, mState.mLength);
+      bool ok = aString.Append(m2b, mState.mLength, mozilla::fallible_t());
+      if (!ok) {
+        return false;
+      }
+
+      return true;
     } else {
-      AppendASCIItoUTF16(Substring(m1b, m1b + mState.mLength),
-                         aString);
+      return AppendASCIItoUTF16(Substring(m1b, mState.mLength), aString,
+                                mozilla::fallible_t());
     }
   }
 
@@ -184,11 +154,32 @@ public:
    * @param aOffset where to start the substring in this text fragment
    * @param aLength the length of the substring
    */
-  void AppendTo(nsAString& aString, PRInt32 aOffset, PRInt32 aLength) const {
+  void AppendTo(nsAString& aString, int32_t aOffset, int32_t aLength) const {
+    if (!AppendTo(aString, aOffset, aLength, mozilla::fallible_t())) {
+      NS_ABORT_OOM(aLength);
+    }
+  }
+
+  /**
+   * Append a substring of the contents of this string fragment to aString.
+   * @param aString the string in which to append
+   * @param aOffset where to start the substring in this text fragment
+   * @param aLength the length of the substring
+   * @return false if an out of memory condition is detected, true otherwise
+   */
+  bool AppendTo(nsAString& aString, int32_t aOffset, int32_t aLength,
+                const mozilla::fallible_t&) const NS_WARN_UNUSED_RESULT
+  {
     if (mState.mIs2b) {
-      aString.Append(m2b + aOffset, aLength);
+      bool ok = aString.Append(m2b + aOffset, aLength, mozilla::fallible_t());
+      if (!ok) {
+        return false;
+      }
+
+      return true;
     } else {
-      AppendASCIItoUTF16(Substring(m1b + aOffset, m1b + aOffset + aLength), aString);
+      return AppendASCIItoUTF16(Substring(m1b + aOffset, aLength), aString,
+                                mozilla::fallible_t());
     }
   }
 
@@ -198,40 +189,31 @@ public:
    * lie within the fragments data. The fragments data is converted if
    * necessary.
    */
-  void CopyTo(PRUnichar *aDest, PRInt32 aOffset, PRInt32 aCount);
+  void CopyTo(char16_t *aDest, int32_t aOffset, int32_t aCount);
 
   /**
    * Return the character in the text-fragment at the given
-   * index. This always returns a PRUnichar.
+   * index. This always returns a char16_t.
    */
-  PRUnichar CharAt(PRInt32 aIndex) const
+  char16_t CharAt(int32_t aIndex) const
   {
-    NS_ASSERTION(PRUint32(aIndex) < mState.mLength, "bad index");
+    NS_ASSERTION(uint32_t(aIndex) < mState.mLength, "bad index");
     return mState.mIs2b ? m2b[aIndex] : static_cast<unsigned char>(m1b[aIndex]);
   }
 
   struct FragmentBits {
-    // PRUint32 to ensure that the values are unsigned, because we
+    // uint32_t to ensure that the values are unsigned, because we
     // want 0/1, not 0/-1!
     // Making these bool causes Windows to not actually pack them,
     // which causes crashes because we assume this structure is no more than
     // 32 bits!
-    PRUint32 mInHeap : 1;
-    PRUint32 mIs2b : 1;
-    PRUint32 mIsBidi : 1;
-    PRUint32 mLength : 29;
+    uint32_t mInHeap : 1;
+    uint32_t mIs2b : 1;
+    uint32_t mIsBidi : 1;
+    uint32_t mLength : 29;
   };
 
-  /**
-   * Returns the size taken in memory by this text fragment.
-   * @return the size taken in memory by this text fragment.
-   */
-  PRInt64 SizeOf() const
-  {
-    PRInt64 size = sizeof(*this);
-    size += GetLength() * (Is2b() ? sizeof(*m2b) : sizeof(*m1b));
-    return size;
-  }
+  size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
 
 private:
   void ReleaseText();
@@ -240,15 +222,15 @@ private:
    * Scan the contents of the fragment and turn on mState.mIsBidi if it
    * includes any Bidi characters.
    */
-  void UpdateBidiFlag(const PRUnichar* aBuffer, PRUint32 aLength);
+  void UpdateBidiFlag(const char16_t* aBuffer, uint32_t aLength);
  
   union {
-    PRUnichar *m2b;
+    char16_t *m2b;
     const char *m1b; // This is const since it can point to shared data
   };
 
   union {
-    PRUint32 mAllBits;
+    uint32_t mAllBits;
     FragmentBits mState;
   };
 };
