@@ -1,55 +1,23 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: sw=2 ts=2 et :
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Plugin App.
- *
- * The Initial Developer of the Original Code is
- *   Ben Turner <bent.mozilla@gmail.com>
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "PluginScriptableObjectParent.h"
-#include "PluginScriptableObjectUtils.h"
 
-#include "nsNPAPIPlugin.h"
+#include "mozilla/DebugOnly.h"
+#include "mozilla/plugins/PluginIdentifierParent.h"
 #include "mozilla/unused.h"
-#include "mozilla/Util.h"
+#include "nsCxPusher.h"
+#include "nsNPAPIPlugin.h"
+#include "PluginScriptableObjectUtils.h"
 
 using namespace mozilla::plugins;
 using namespace mozilla::plugins::parent;
 
 namespace {
 
-typedef PluginIdentifierParent::StackIdentifier StackIdentifier;
-
 inline void
 ReleaseVariant(NPVariant& aVariant,
                PluginInstanceParent* aInstance)
@@ -69,7 +37,7 @@ PluginScriptableObjectParent::ScriptableAllocate(NPP aInstance,
 {
   if (aClass != GetClass()) {
     NS_ERROR("Huh?! Wrong class!");
-    return nsnull;
+    return nullptr;
   }
 
   return new ParentNPObject();
@@ -138,7 +106,7 @@ PluginScriptableObjectParent::ScriptableHasMethod(NPObject* aObject,
     return false;
   }
 
-  StackIdentifier identifier(aObject, aName);
+  PluginIdentifierParent::StackIdentifier identifier(aObject, aName);
   if (!identifier) {
     return false;
   }
@@ -178,7 +146,7 @@ PluginScriptableObjectParent::ScriptableInvoke(NPObject* aObject,
     return false;
   }
 
-  StackIdentifier identifier(aObject, aName);
+  PluginIdentifierParent::StackIdentifier identifier(aObject, aName);
   if (!identifier) {
     return false;
   }
@@ -280,7 +248,7 @@ PluginScriptableObjectParent::ScriptableHasProperty(NPObject* aObject,
     return false;
   }
 
-  StackIdentifier identifier(aObject, aName);
+  PluginIdentifierParent::StackIdentifier identifier(aObject, aName);
   if (!identifier) {
     return false;
   }
@@ -329,7 +297,7 @@ PluginScriptableObjectParent::ScriptableSetProperty(NPObject* aObject,
     return false;
   }
 
-  StackIdentifier identifier(aObject, aName);
+  PluginIdentifierParent::StackIdentifier identifier(aObject, aName);
   if (!identifier) {
     return false;
   }
@@ -372,7 +340,7 @@ PluginScriptableObjectParent::ScriptableRemoveProperty(NPObject* aObject,
     return false;
   }
 
-  StackIdentifier identifier(aObject, aName);
+  PluginIdentifierParent::StackIdentifier identifier(aObject, aName);
   if (!identifier) {
     return false;
   }
@@ -431,7 +399,7 @@ PluginScriptableObjectParent::ScriptableEnumerate(NPObject* aObject,
 
   *aCount = identifiers.Length();
   if (!*aCount) {
-    *aIdentifiers = nsnull;
+    *aIdentifiers = nullptr;
     return true;
   }
 
@@ -441,7 +409,7 @@ PluginScriptableObjectParent::ScriptableEnumerate(NPObject* aObject,
     return false;
   }
 
-  for (PRUint32 index = 0; index < *aCount; index++) {
+  for (uint32_t index = 0; index < *aCount; index++) {
     PluginIdentifierParent* id =
       static_cast<PluginIdentifierParent*>(identifiers[index]);
     (*aIdentifiers)[index] = id->ToNPIdentifier();
@@ -516,8 +484,8 @@ const NPClass PluginScriptableObjectParent::sNPClass = {
 
 PluginScriptableObjectParent::PluginScriptableObjectParent(
                                                      ScriptableObjectType aType)
-: mInstance(nsnull),
-  mObject(nsnull),
+: mInstance(nullptr),
+  mObject(nullptr),
   mProtectCount(0),
   mType(aType)
 {
@@ -528,7 +496,7 @@ PluginScriptableObjectParent::~PluginScriptableObjectParent()
   if (mObject) {
     if (mObject->_class == GetClass()) {
       NS_ASSERTION(mType == Proxy, "Wrong type!");
-      static_cast<ParentNPObject*>(mObject)->parent = nsnull;
+      static_cast<ParentNPObject*>(mObject)->parent = nullptr;
     }
     else {
       NS_ASSERTION(mType == LocalObject, "Wrong type!");
@@ -628,7 +596,7 @@ PluginScriptableObjectParent::GetObject(bool aCanResurrect)
 {
   if (!mObject && aCanResurrect && !ResurrectProxyObject()) {
     NS_ERROR("Null object!");
-    return nsnull;
+    return nullptr;
   }
   return mObject;
 }
@@ -670,7 +638,7 @@ PluginScriptableObjectParent::DropNPObject()
   NS_ASSERTION(instance, "Must have an instance!");
 
   instance->UnregisterNPObject(mObject);
-  mObject = nsnull;
+  mObject = nullptr;
 
   unused << SendUnprotect();
 }
@@ -739,8 +707,8 @@ PluginScriptableObjectParent::AnswerInvoke(PPluginIdentifierParent* aId,
     return true;
   }
 
-  nsAutoTArray<NPVariant, 10> convertedArgs;
-  PRUint32 argCount = aArgs.Length();
+  AutoFallibleTArray<NPVariant, 10> convertedArgs;
+  uint32_t argCount = aArgs.Length();
 
   if (!convertedArgs.SetLength(argCount)) {
     *aResult = void_t();
@@ -748,7 +716,7 @@ PluginScriptableObjectParent::AnswerInvoke(PPluginIdentifierParent* aId,
     return true;
   }
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     if (!ConvertToVariant(aArgs[index], convertedArgs[index], instance)) {
       // Don't leak things we've already converted!
       while (index-- > 0) {
@@ -765,7 +733,7 @@ PluginScriptableObjectParent::AnswerInvoke(PPluginIdentifierParent* aId,
   bool success = npn->invoke(instance->GetNPP(), mObject, id->ToNPIdentifier(),
                              convertedArgs.Elements(), argCount, &result);
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     ReleaseVariant(convertedArgs[index], instance);
   }
 
@@ -822,8 +790,8 @@ PluginScriptableObjectParent::AnswerInvokeDefault(const InfallibleTArray<Variant
     return true;
   }
 
-  nsAutoTArray<NPVariant, 10> convertedArgs;
-  PRUint32 argCount = aArgs.Length();
+  AutoFallibleTArray<NPVariant, 10> convertedArgs;
+  uint32_t argCount = aArgs.Length();
 
   if (!convertedArgs.SetLength(argCount)) {
     *aResult = void_t();
@@ -831,7 +799,7 @@ PluginScriptableObjectParent::AnswerInvokeDefault(const InfallibleTArray<Variant
     return true;
   }
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     if (!ConvertToVariant(aArgs[index], convertedArgs[index], instance)) {
       // Don't leak things we've already converted!
       while (index-- > 0) {
@@ -848,7 +816,7 @@ PluginScriptableObjectParent::AnswerInvokeDefault(const InfallibleTArray<Variant
                                     convertedArgs.Elements(), argCount,
                                     &result);
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     ReleaseVariant(convertedArgs[index], instance);
   }
 
@@ -1069,20 +1037,14 @@ PluginScriptableObjectParent::AnswerEnumerate(InfallibleTArray<PPluginIdentifier
     return true;
   }
 
-  if (!aProperties->SetCapacity(idCount)) {
-    npn->memfree(ids);
-    *aSuccess = false;
-    return true;
-  }
-
-  JSContext* cx = GetJSContext(instance->GetNPP());
-  JSAutoRequest ar(cx);
+  aProperties->SetCapacity(idCount);
 
+  mozilla::AutoSafeJSContext cx;
   for (uint32_t index = 0; index < idCount; index++) {
     // Because of GC hazards, all identifiers returned from enumerate
     // must be made permanent.
     if (_identifierisstring(ids[index])) {
-      JSString* str = NPIdentifierToString(ids[index]);
+      JS::Rooted<JSString*> str(cx, NPIdentifierToString(ids[index]));
       if (!JS_StringHasBeenInterned(cx, str)) {
         DebugOnly<JSString*> str2 = JS_InternJSString(cx, str);
         NS_ASSERTION(str2 == str, "Interning a JS string which is currently an ID should return itself.");
@@ -1130,8 +1092,8 @@ PluginScriptableObjectParent::AnswerConstruct(const InfallibleTArray<Variant>& a
     return true;
   }
 
-  nsAutoTArray<NPVariant, 10> convertedArgs;
-  PRUint32 argCount = aArgs.Length();
+  AutoFallibleTArray<NPVariant, 10> convertedArgs;
+  uint32_t argCount = aArgs.Length();
 
   if (!convertedArgs.SetLength(argCount)) {
     *aResult = void_t();
@@ -1139,7 +1101,7 @@ PluginScriptableObjectParent::AnswerConstruct(const InfallibleTArray<Variant>& a
     return true;
   }
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     if (!ConvertToVariant(aArgs[index], convertedArgs[index], instance)) {
       // Don't leak things we've already converted!
       while (index-- > 0) {
@@ -1155,7 +1117,7 @@ PluginScriptableObjectParent::AnswerConstruct(const InfallibleTArray<Variant>& a
   bool success = npn->construct(instance->GetNPP(), mObject,
                                 convertedArgs.Elements(), argCount, &result);
 
-  for (PRUint32 index = 0; index < argCount; index++) {
+  for (uint32_t index = 0; index < argCount; index++) {
     ReleaseVariant(convertedArgs[index], instance);
   }
 
@@ -1248,10 +1210,10 @@ PluginScriptableObjectParent::AnswerNPN_Evaluate(const nsCString& aScript,
   return true;
 }
 
-JSBool
+bool
 PluginScriptableObjectParent::GetPropertyHelper(NPIdentifier aName,
-                                                PRBool* aHasProperty,
-                                                PRBool* aHasMethod,
+                                                bool* aHasProperty,
+                                                bool* aHasMethod,
                                                 NPVariant* aResult)
 {
   NS_ASSERTION(Type() == Proxy, "Bad type!");
@@ -1259,31 +1221,31 @@ PluginScriptableObjectParent::GetPropertyHelper(NPIdentifier aName,
   ParentNPObject* object = static_cast<ParentNPObject*>(mObject);
   if (object->invalidated) {
     NS_WARNING("Calling method on an invalidated object!");
-    return JS_FALSE;
+    return false;
   }
 
-  StackIdentifier identifier(GetInstance(), aName);
+  PluginIdentifierParent::StackIdentifier identifier(GetInstance(), aName);
   if (!identifier) {
-    return JS_FALSE;
+    return false;
   }
 
   bool hasProperty, hasMethod, success;
   Variant result;
   if (!CallGetChildProperty(identifier, &hasProperty, &hasMethod, &result,
                             &success)) {
-    return JS_FALSE;
+    return false;
   }
 
   if (!success) {
-    return JS_FALSE;
+    return false;
   }
 
   if (!ConvertToVariant(result, *aResult, GetInstance())) {
     NS_WARNING("Failed to convert result!");
-    return JS_FALSE;
+    return false;
   }
 
   *aHasProperty = hasProperty;
   *aHasMethod = hasMethod;
-  return JS_TRUE;
+  return true;
 }
