@@ -1,54 +1,21 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 sw=2 et tw=79: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2007
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Dave Camp <dcamp@mozilla.com>
- *   Robert Sayre <sayrer@gmail.com>
- *   Nils Maier <maierman@web.de>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "jsapi.h"
-#include "jsdbgapi.h"
-#include "nsIServiceManager.h"
+#include "js/CharacterEncoding.h"
+#include "js/OldDebugAPI.h"
 #include "nsJSON.h"
 #include "nsIXPConnect.h"
 #include "nsIXPCScriptable.h"
 #include "nsStreamUtils.h"
 #include "nsIInputStream.h"
 #include "nsStringStream.h"
-#include "nsICharsetConverterManager.h"
+#include "mozilla/dom/EncodingUtils.h"
+#include "nsIUnicodeEncoder.h"
+#include "nsIUnicodeDecoder.h"
 #include "nsXPCOMStrings.h"
 #include "nsNetUtil.h"
 #include "nsContentUtils.h"
@@ -56,8 +23,10 @@
 #include "nsCRTGlue.h"
 #include "nsAutoPtr.h"
 #include "nsIScriptSecurityManager.h"
+#include "mozilla/Maybe.h"
+#include <algorithm>
 
-static const char kXPConnectServiceCID[] = "@mozilla.org/js/xpc/XPConnect;1";
+using mozilla::dom::EncodingUtils;
 
 #define JSON_STREAM_BUFSIZE 4096
 
@@ -82,27 +51,31 @@ enum DeprecationWarning { EncodeWarning, DecodeWarning };
 static nsresult
 WarnDeprecatedMethod(DeprecationWarning warning)
 {
-  return nsContentUtils::ReportToConsole(nsContentUtils::eDOM_PROPERTIES,
+  return nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                         NS_LITERAL_CSTRING("DOM Core"), nullptr,
+                                         nsContentUtils::eDOM_PROPERTIES,
                                          warning == EncodeWarning
                                          ? "nsIJSONEncodeDeprecatedWarning"
-                                         : "nsIJSONDecodeDeprecatedWarning",
-                                         nsnull, 0,
-                                         nsnull,
-                                         EmptyString(), 0, 0,
-                                         nsIScriptError::warningFlag,
-                                         "DOM Core");
+                                         : "nsIJSONDecodeDeprecatedWarning");
 }
 
 NS_IMETHODIMP
-nsJSON::Encode(nsAString &aJSON)
+nsJSON::Encode(JS::Handle<JS::Value> aValue, JSContext* cx, uint8_t aArgc,
+               nsAString &aJSON)
 {
   // This function should only be called from JS.
   nsresult rv = WarnDeprecatedMethod(EncodeWarning);
   if (NS_FAILED(rv))
     return rv;
 
+  if (aArgc == 0) {
+    aJSON.Truncate();
+    aJSON.SetIsVoid(true);
+    return NS_OK;
+  }
+
   nsJSONWriter writer;
-  rv = EncodeInternal(&writer);
+  rv = EncodeInternal(cx, aValue, &writer);
 
   // FIXME: bug 408838. Get exception types sorted out
   if (NS_SUCCEEDED(rv) || rv == NS_ERROR_INVALID_ARG) {
@@ -110,7 +83,7 @@ nsJSON::Encode(nsAString &aJSON)
     // if we didn't consume anything, it's not JSON, so return null
     if (!writer.DidWrite()) {
       aJSON.Truncate();
-      aJSON.SetIsVoid(PR_TRUE);
+      aJSON.SetIsVoid(true);
     } else {
       writer.FlushBuffer();
       aJSON.Append(writer.mOutputString);
@@ -139,7 +112,10 @@ static nsresult CheckCharset(const char* aCharset)
 NS_IMETHODIMP
 nsJSON::EncodeToStream(nsIOutputStream *aStream,
                        const char* aCharset,
-                       const PRBool aWriteBOM)
+                       const bool aWriteBOM,
+                       JS::Handle<JS::Value> val,
+                       JSContext* cx,
+                       uint8_t aArgc)
 {
   // This function should only be called from JS.
   NS_ENSURE_ARG(aStream);
@@ -159,7 +135,7 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
   //  aStream = bufferedStream;
   //}
 
-  PRUint32 ignored;
+  uint32_t ignored;
   if (aWriteBOM) {
     if (strcmp(aCharset, "UTF-8") == 0)
       rv = aStream->Write(UTF8BOM, 3, &ignored);
@@ -174,7 +150,11 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
   rv = writer.SetCharset(aCharset);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = EncodeInternal(&writer);
+  if (aArgc == 0) {
+    return NS_OK;
+  }
+
+  rv = EncodeInternal(cx, val, &writer);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bufferedStream->Flush();
@@ -182,54 +162,34 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
   return rv;
 }
 
-static JSBool
-WriteCallback(const jschar *buf, uint32 len, void *data)
+static bool
+WriteCallback(const jschar *buf, uint32_t len, void *data)
 {
   nsJSONWriter *writer = static_cast<nsJSONWriter*>(data);
-  nsresult rv =  writer->Write((const PRUnichar*)buf, (PRUint32)len);
+  nsresult rv =  writer->Write((const char16_t*)buf, (uint32_t)len);
   if (NS_FAILED(rv))
-    return JS_FALSE;
+    return false;
 
-  return JS_TRUE;
+  return true;
 }
 
 NS_IMETHODIMP
-nsJSON::EncodeFromJSVal(jsval *value, JSContext *cx, nsAString &result)
+nsJSON::EncodeFromJSVal(JS::Value *value, JSContext *cx, nsAString &result)
 {
   result.Truncate();
 
-  // Begin a new request
-  JSAutoRequest ar(cx);
-
-  JSAutoEnterCompartment ac;
-  JSObject *obj;
-  nsIScriptSecurityManager *ssm = nsnull;
-  if (JSVAL_IS_OBJECT(*value) && (obj = JSVAL_TO_OBJECT(*value))) {
-    if (!ac.enter(cx, obj)) {
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIPrincipal> principal;
-    ssm = nsContentUtils::GetSecurityManager();
-    nsresult rv = ssm->GetObjectPrincipal(cx, obj, getter_AddRefs(principal));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    JSStackFrame *fp = nsnull;
-    rv = ssm->PushContextPrincipal(cx, JS_FrameIterator(cx, &fp), principal);
-    NS_ENSURE_SUCCESS(rv, rv);
+  mozilla::Maybe<JSAutoCompartment> ac;
+  if (value->isObject()) {
+    JS::Rooted<JSObject*> obj(cx, &value->toObject());
+    ac.construct(cx, obj);
   }
 
   nsJSONWriter writer;
-  JSBool ok = JS_Stringify(cx, value, NULL, JSVAL_NULL,
-                           WriteCallback, &writer);
-
-  if (ssm) {
-    ssm->PopContextPrincipal(cx);
-  }
-
-  if (!ok) {
+  JS::Rooted<JS::Value> vp(cx, *value);
+  if (!JS_Stringify(cx, &vp, JS::NullPtr(), JS::NullHandleValue, WriteCallback, &writer)) {
     return NS_ERROR_XPC_BAD_CONVERT_JS;
   }
+  *value = vp;
 
   NS_ENSURE_TRUE(writer.DidWrite(), NS_ERROR_UNEXPECTED);
   writer.FlushBuffer();
@@ -238,52 +198,15 @@ nsJSON::EncodeFromJSVal(jsval *value, JSContext *cx, nsAString &result)
 }
 
 nsresult
-nsJSON::EncodeInternal(nsJSONWriter *writer)
+nsJSON::EncodeInternal(JSContext* cx, const JS::Value& aValue,
+                       nsJSONWriter* writer)
 {
-  nsresult rv;
-  nsIXPConnect *xpc = nsContentUtils::XPConnect();
-  if (!xpc)
-    return NS_ERROR_FAILURE;
-
-  // Now fish for the JS argument. If it's a call to encode, we'll
-  // want the first argument. If it's a call to encodeToStream,
-  // we'll want the forth.
-  const PRUint32 firstArg = writer->mStream ? 3 : 0;
-
-  nsAXPCNativeCallContext *cc = nsnull;
-  rv = xpc->GetCurrentNativeCallContext(&cc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  PRUint32 argc = 0;
-  rv = cc->GetArgc(&argc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // If the argument wasn't provided, there's nothing to serialize.
-  if (argc <= firstArg)
-    return NS_OK;
-
-  JSContext *cx = nsnull;
-  rv = cc->GetJSContext(&cx);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSAutoRequest ar(cx);
-
-  // Get the object we're going to serialize.
-  jsval *argv = nsnull;
-  rv = cc->GetArgvPtr(&argv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // If the argument wasn't provided, there's nothing to serialize.
-  if (argc <= firstArg)
-    return NS_ERROR_INVALID_ARG;
-
-  jsval *vp = &argv[firstArg];
-  
   // Backward compatibility:
   // nsIJSON does not allow to serialize anything other than objects
-  JSObject *obj;
-  if (!JSVAL_IS_OBJECT(*vp) || !(obj = JSVAL_TO_OBJECT(*vp)))
+  if (!aValue.isObject()) {
     return NS_ERROR_INVALID_ARG;
+  }
+  JS::Rooted<JSObject*> obj(cx, &aValue.toObject());
 
   /* Backward compatibility:
    * Manually call toJSON if implemented by the object and check that
@@ -291,13 +214,13 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
    * Note: It is perfectly fine to not implement toJSON, so it is
    * perfectly fine for GetMethod to fail
    */
-  jsval toJSON;
-  if (JS_GetMethod(cx, obj, "toJSON", NULL, &toJSON) &&
-      !JSVAL_IS_PRIMITIVE(toJSON) &&
-      JS_ObjectIsCallable(cx, JSVAL_TO_OBJECT(toJSON))) {
-
+  JS::Rooted<JS::Value> val(cx, aValue);
+  JS::Rooted<JS::Value> toJSON(cx);
+  if (JS_GetProperty(cx, obj, "toJSON", &toJSON) &&
+      toJSON.isObject() &&
+      JS_ObjectIsCallable(cx, &toJSON.toObject())) {
     // If toJSON is implemented, it must not throw
-    if (!JS_CallFunctionValue(cx, obj, toJSON, 0, NULL, vp)) {
+    if (!JS_CallFunctionValue(cx, obj, toJSON, JS::HandleValueArray::empty(), &val)) {
       if (JS_IsExceptionPending(cx))
         // passing NS_OK will throw the pending exception
         return NS_OK;
@@ -308,7 +231,7 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
 
     // Backward compatibility:
     // nsIJSON does not allow to serialize anything other than objects
-    if (JSVAL_IS_PRIMITIVE(*vp))
+    if (val.isPrimitive())
       return NS_ERROR_INVALID_ARG;
   }
   // GetMethod may have thrown
@@ -317,32 +240,32 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
     return NS_OK;
 
   // Backward compatibility:
-  // function/xml shall not pass, just "plain" objects and arrays
-  JSType type = JS_TypeOfValue(cx, *vp);
-  if (type == JSTYPE_FUNCTION || type == JSTYPE_XML)
+  // function shall not pass, just "plain" objects and arrays
+  JSType type = JS_TypeOfValue(cx, val);
+  if (type == JSTYPE_FUNCTION)
     return NS_ERROR_INVALID_ARG;
 
   // We're good now; try to stringify
-  if (!JS_Stringify(cx, vp, NULL, JSVAL_NULL, WriteCallback, writer))
+  if (!JS_Stringify(cx, &val, JS::NullPtr(), JS::NullHandleValue, WriteCallback, writer))
     return NS_ERROR_FAILURE;
 
   return NS_OK;
 }
 
 
-nsJSONWriter::nsJSONWriter() : mStream(nsnull),
-                               mBuffer(nsnull),
+nsJSONWriter::nsJSONWriter() : mStream(nullptr),
+                               mBuffer(nullptr),
                                mBufferCount(0),
-                               mDidWrite(PR_FALSE),
-                               mEncoder(nsnull)
+                               mDidWrite(false),
+                               mEncoder(nullptr)
 {
 }
 
 nsJSONWriter::nsJSONWriter(nsIOutputStream *aStream) : mStream(aStream),
-                                                       mBuffer(nsnull),
+                                                       mBuffer(nullptr),
                                                        mBufferCount(0),
-                                                       mDidWrite(PR_FALSE),
-                                                       mEncoder(nsnull)
+                                                       mDidWrite(false),
+                                                       mEncoder(nullptr)
 {
 }
 
@@ -356,13 +279,9 @@ nsJSONWriter::SetCharset(const char* aCharset)
 {
   nsresult rv = NS_OK;
   if (mStream) {
-    nsCOMPtr<nsICharsetConverterManager> ccm =
-      do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = ccm->GetUnicodeEncoder(aCharset, getter_AddRefs(mEncoder));
-    NS_ENSURE_SUCCESS(rv, rv);
+    mEncoder = EncodingUtils::EncoderForEncoding(aCharset);
     rv = mEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Signal,
-                                          nsnull, nsnull);
+                                          nullptr, '\0');
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -370,17 +289,17 @@ nsJSONWriter::SetCharset(const char* aCharset)
 }
 
 nsresult
-nsJSONWriter::Write(const PRUnichar *aBuffer, PRUint32 aLength)
+nsJSONWriter::Write(const char16_t *aBuffer, uint32_t aLength)
 {
   if (mStream) {
     return WriteToStream(mStream, mEncoder, aBuffer, aLength);
   }
 
   if (!mDidWrite) {
-    mBuffer = new PRUnichar[JSON_STREAM_BUFSIZE];
+    mBuffer = new char16_t[JSON_STREAM_BUFSIZE];
     if (!mBuffer)
       return NS_ERROR_OUT_OF_MEMORY;
-    mDidWrite = PR_TRUE;
+    mDidWrite = true;
   }
 
   if (JSON_STREAM_BUFSIZE <= aLength + mBufferCount) {
@@ -392,14 +311,14 @@ nsJSONWriter::Write(const PRUnichar *aBuffer, PRUint32 aLength)
     // we know mBufferCount is 0 because we know we hit the if above
     mOutputString.Append(aBuffer, aLength);
   } else {
-    memcpy(&mBuffer[mBufferCount], aBuffer, aLength * sizeof(PRUnichar));
+    memcpy(&mBuffer[mBufferCount], aBuffer, aLength * sizeof(char16_t));
     mBufferCount += aLength;
   }
 
   return NS_OK;
 }
 
-PRBool nsJSONWriter::DidWrite()
+bool nsJSONWriter::DidWrite()
 {
   return mDidWrite;
 }
@@ -413,15 +332,15 @@ nsJSONWriter::FlushBuffer()
 nsresult
 nsJSONWriter::WriteToStream(nsIOutputStream *aStream,
                             nsIUnicodeEncoder *encoder,
-                            const PRUnichar *aBuffer,
-                            PRUint32 aLength)
+                            const char16_t *aBuffer,
+                            uint32_t aLength)
 {
   nsresult rv;
-  PRInt32 srcLength = aLength;
-  PRUint32 bytesWritten;
+  int32_t srcLength = aLength;
+  uint32_t bytesWritten;
 
-  // The bytes written to the stream might differ from the PRUnichar size
-  PRInt32 aDestLength;
+  // The bytes written to the stream might differ from the char16_t size
+  int32_t aDestLength;
   rv = encoder->GetMaxLength(aBuffer, srcLength, &aDestLength);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -435,73 +354,55 @@ nsJSONWriter::WriteToStream(nsIOutputStream *aStream,
     rv = aStream->Write(destBuf, aDestLength, &bytesWritten);
 
   NS_Free(destBuf);
-  mDidWrite = PR_TRUE;
+  mDidWrite = true;
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsJSON::Decode(const nsAString& json)
+nsJSON::Decode(const nsAString& json, JSContext* cx,
+               JS::MutableHandle<JS::Value> aRetval)
 {
   nsresult rv = WarnDeprecatedMethod(DecodeWarning);
   if (NS_FAILED(rv))
     return rv;
 
-  const PRUnichar *data;
-  PRUint32 len = NS_StringGetData(json, &data);
+  const char16_t *data;
+  uint32_t len = NS_StringGetData(json, &data);
   nsCOMPtr<nsIInputStream> stream;
   rv = NS_NewByteInputStream(getter_AddRefs(stream),
                              reinterpret_cast<const char*>(data),
-                             len * sizeof(PRUnichar),
+                             len * sizeof(char16_t),
                              NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
-  return DecodeInternal(stream, len, PR_FALSE);
+  return DecodeInternal(cx, stream, len, false, aRetval);
 }
 
 NS_IMETHODIMP
-nsJSON::DecodeFromStream(nsIInputStream *aStream, PRInt32 aContentLength)
+nsJSON::DecodeFromStream(nsIInputStream *aStream, int32_t aContentLength,
+                         JSContext* cx, JS::MutableHandle<JS::Value> aRetval)
 {
-  return DecodeInternal(aStream, aContentLength, PR_TRUE);
+  return DecodeInternal(cx, aStream, aContentLength, true, aRetval);
 }
 
 NS_IMETHODIMP
-nsJSON::DecodeToJSVal(const nsAString &str, JSContext *cx, jsval *result)
+nsJSON::DecodeToJSVal(const nsAString &str, JSContext *cx,
+                      JS::MutableHandle<JS::Value> result)
 {
-  JSAutoRequest ar(cx);
-
-  if (!JS_ParseJSON(cx, (jschar*)PromiseFlatString(str).get(),
-                    (uint32)str.Length(), result)) {
+  if (!JS_ParseJSON(cx, static_cast<const jschar*>(PromiseFlatString(str).get()),
+                    str.Length(), result)) {
     return NS_ERROR_UNEXPECTED;
   }
-
   return NS_OK;
 }
 
 nsresult
-nsJSON::DecodeInternal(nsIInputStream *aStream,
-                       PRInt32 aContentLength,
-                       PRBool aNeedsConverter,
-                       DecodingMode mode /* = STRICT */)
+nsJSON::DecodeInternal(JSContext* cx,
+                       nsIInputStream *aStream,
+                       int32_t aContentLength,
+                       bool aNeedsConverter,
+                       JS::MutableHandle<JS::Value> aRetval)
 {
-  nsresult rv;
-  nsIXPConnect *xpc = nsContentUtils::XPConnect();
-  if (!xpc)
-    return NS_ERROR_FAILURE;
-
-  nsAXPCNativeCallContext *cc = nsnull;
-  rv = xpc->GetCurrentNativeCallContext(&cc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  jsval *retvalPtr;
-  rv = cc->GetRetValPtr(&retvalPtr);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSContext *cx = nsnull;
-  rv = cc->GetJSContext(&cx);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSAutoRequest ar(cx);
-
   // Consume the stream
   nsCOMPtr<nsIChannel> jsonChannel;
   if (!mURI) {
@@ -510,19 +411,17 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
       return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  rv = NS_NewInputStreamChannel(getter_AddRefs(jsonChannel), mURI, aStream,
-                                NS_LITERAL_CSTRING("application/json"));
+  nsresult rv =
+    NS_NewInputStreamChannel(getter_AddRefs(jsonChannel), mURI, aStream,
+                             NS_LITERAL_CSTRING("application/json"));
   if (!jsonChannel || NS_FAILED(rv))
     return NS_ERROR_FAILURE;
 
-  nsRefPtr<nsJSONListener>
-    jsonListener(new nsJSONListener(cx, retvalPtr, aNeedsConverter, mode));
-
-  if (!jsonListener)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsRefPtr<nsJSONListener> jsonListener =
+    new nsJSONListener(cx, aRetval.address(), aNeedsConverter);
 
   //XXX this stream pattern should be consolidated in netwerk
-  rv = jsonListener->OnStartRequest(jsonChannel, nsnull);
+  rv = jsonListener->OnStartRequest(jsonChannel, nullptr);
   if (NS_FAILED(rv)) {
     jsonChannel->Cancel(rv);
     return rv;
@@ -530,9 +429,9 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
 
   nsresult status;
   jsonChannel->GetStatus(&status);
-  PRUint32 offset = 0;
+  uint64_t offset = 0;
   while (NS_SUCCEEDED(status)) {
-    PRUint32 available;
+    uint64_t available;
     rv = aStream->Available(&available);
     if (rv == NS_BASE_STREAM_CLOSED) {
       rv = NS_OK;
@@ -545,8 +444,13 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
     if (!available)
       break; // blocking input stream has none available when done
 
-    rv = jsonListener->OnDataAvailable(jsonChannel, nsnull,
-                                       aStream, offset, available);
+    if (available > UINT32_MAX)
+      available = UINT32_MAX;
+
+    rv = jsonListener->OnDataAvailable(jsonChannel, nullptr,
+                                       aStream,
+                                       offset,
+                                       (uint32_t)available);
     if (NS_FAILED(rv)) {
       jsonChannel->Cancel(rv);
       break;
@@ -557,47 +461,9 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = jsonListener->OnStopRequest(jsonChannel, nsnull, status);
+  rv = jsonListener->OnStopRequest(jsonChannel, nullptr, status);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = cc->SetReturnValueWasSet(PR_TRUE);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsJSON::LegacyDecode(const nsAString& json)
-{
-  const PRUnichar *data;
-  PRUint32 len = NS_StringGetData(json, &data);
-  nsCOMPtr<nsIInputStream> stream;
-  nsresult rv = NS_NewByteInputStream(getter_AddRefs(stream),
-                                      (const char*) data,
-                                      len * sizeof(PRUnichar),
-                                      NS_ASSIGNMENT_DEPEND);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return DecodeInternal(stream, len, PR_FALSE, LEGACY);
-}
-
-NS_IMETHODIMP
-nsJSON::LegacyDecodeFromStream(nsIInputStream *aStream, PRInt32 aContentLength)
-{
-  return DecodeInternal(aStream, aContentLength, PR_TRUE, LEGACY);
-}
-
-NS_IMETHODIMP
-nsJSON::LegacyDecodeToJSVal(const nsAString &str, JSContext *cx, jsval *result)
-{
-  JSAutoRequest ar(cx);
-
-  if (!js::ParseJSONWithReviver(cx, (jschar*)PromiseFlatString(str).get(),
-                                (uint32)str.Length(), js::NullValue(),
-                                result, LEGACY)) {
-    return NS_ERROR_UNEXPECTED;
-  }
-
   return NS_OK;
 }
 
@@ -614,13 +480,11 @@ NS_NewJSON(nsISupports* aOuter, REFNSIID aIID, void** aResult)
   return NS_OK;
 }
 
-nsJSONListener::nsJSONListener(JSContext *cx, jsval *rootVal,
-                               PRBool needsConverter,
-                               DecodingMode mode /* = STRICT */)
-  : mNeedsConverter(needsConverter), 
+nsJSONListener::nsJSONListener(JSContext *cx, JS::Value *rootVal,
+                               bool needsConverter)
+  : mNeedsConverter(needsConverter),
     mCx(cx),
-    mRootVal(rootVal),
-    mDecodingMode(mode)
+    mRootVal(rootVal)
 {
 }
 
@@ -641,7 +505,7 @@ NS_IMETHODIMP
 nsJSONListener::OnStartRequest(nsIRequest *aRequest, nsISupports *aContext)
 {
   mSniffBuffer.Truncate();
-  mDecoder = nsnull;
+  mDecoder = nullptr;
 
   return NS_OK;
 }
@@ -655,15 +519,19 @@ nsJSONListener::OnStopRequest(nsIRequest *aRequest, nsISupports *aContext,
   // This can happen with short UTF-8 messages (<4 bytes)
   if (!mSniffBuffer.IsEmpty()) {
     // Just consume mSniffBuffer
-    rv = ProcessBytes(nsnull, 0);
+    rv = ProcessBytes(nullptr, 0);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  const jschar* chars = reinterpret_cast<const jschar*>(mBufferedChars.Elements());
-  JSBool ok = js::ParseJSONWithReviver(mCx, chars,
-                                       (uint32) mBufferedChars.Length(),
-                                       js::NullValue(), mRootVal,
-                                       mDecodingMode);
+  JS::Rooted<JS::Value> reviver(mCx, JS::NullValue()), value(mCx);
+
+  JS::ConstTwoByteChars chars(reinterpret_cast<const jschar*>(mBufferedChars.Elements()),
+                              mBufferedChars.Length());
+  bool ok = JS_ParseJSONWithReviver(mCx, chars.get(),
+                                      uint32_t(mBufferedChars.Length()),
+                                      reviver, &value);
+
+  *mRootVal = value;
   mBufferedChars.TruncateLength(0);
   return ok ? NS_OK : NS_ERROR_FAILURE;
 }
@@ -671,27 +539,25 @@ nsJSONListener::OnStopRequest(nsIRequest *aRequest, nsISupports *aContext,
 NS_IMETHODIMP
 nsJSONListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
                                 nsIInputStream *aStream,
-                                PRUint32 aOffset, PRUint32 aLength)
+                                uint64_t aOffset, uint32_t aLength)
 {
-  PRUint32 contentLength;
-  aStream->Available(&contentLength);
   nsresult rv = NS_OK;
 
   if (mNeedsConverter && mSniffBuffer.Length() < 4) {
-    PRUint32 readCount = (aLength < 4) ? aLength : 4;
+    uint32_t readCount = (aLength < 4) ? aLength : 4;
     rv = NS_ConsumeStream(aStream, readCount, mSniffBuffer);
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (mSniffBuffer.Length() < 4)
       return NS_OK;
   }
-  
+
   char buffer[JSON_STREAM_BUFSIZE];
   unsigned long bytesRemaining = aLength - mSniffBuffer.Length();
   while (bytesRemaining) {
     unsigned int bytesRead;
     rv = aStream->Read(buffer,
-                       NS_MIN((unsigned long)sizeof(buffer), bytesRemaining),
+                       std::min((unsigned long)sizeof(buffer), bytesRemaining),
                        &bytesRead);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = ProcessBytes(buffer, bytesRead);
@@ -703,11 +569,11 @@ nsJSONListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
 }
 
 nsresult
-nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
+nsJSONListener::ProcessBytes(const char* aBuffer, uint32_t aByteLength)
 {
   nsresult rv;
   // Check for BOM, or sniff charset
-  nsCAutoString charset;
+  nsAutoCString charset;
   if (mNeedsConverter && !mDecoder) {
     if (!nsContentUtils::CheckForBOM((const unsigned char*) mSniffBuffer.get(),
                                       mSniffBuffer.Length(), charset)) {
@@ -734,11 +600,7 @@ nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
     // We should have a unicode charset by now
     rv = CheckCharset(charset.get());
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCOMPtr<nsICharsetConverterManager> ccm =
-        do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = ccm->GetUnicodeDecoderRaw(charset.get(), getter_AddRefs(mDecoder));
-    NS_ENSURE_SUCCESS(rv, rv);
+    mDecoder = EncodingUtils::DecoderForEncoding(charset);
 
     // consume the sniffed bytes
     rv = ConsumeConverted(mSniffBuffer.get(), mSniffBuffer.Length());
@@ -752,25 +614,25 @@ nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
   if (mNeedsConverter) {
     rv = ConsumeConverted(aBuffer, aByteLength);
   } else {
-    PRUint32 unichars = aByteLength / sizeof(PRUnichar);
-    rv = Consume((PRUnichar *) aBuffer, unichars);
+    uint32_t unichars = aByteLength / sizeof(char16_t);
+    rv = Consume((char16_t *) aBuffer, unichars);
   }
 
   return rv;
 }
 
 nsresult
-nsJSONListener::ConsumeConverted(const char* aBuffer, PRUint32 aByteLength)
+nsJSONListener::ConsumeConverted(const char* aBuffer, uint32_t aByteLength)
 {
   nsresult rv;
-  PRInt32 unicharLength = 0;
-  PRInt32 srcLen = aByteLength;
+  int32_t unicharLength = 0;
+  int32_t srcLen = aByteLength;
 
   rv = mDecoder->GetMaxLength(aBuffer, srcLen, &unicharLength);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUnichar* endelems = mBufferedChars.AppendElements(unicharLength);
-  PRInt32 preLength = unicharLength;
+  char16_t* endelems = mBufferedChars.AppendElements(unicharLength);
+  int32_t preLength = unicharLength;
   rv = mDecoder->Convert(aBuffer, &srcLen, endelems, &unicharLength);
   if (NS_FAILED(rv))
     return rv;
@@ -781,7 +643,7 @@ nsJSONListener::ConsumeConverted(const char* aBuffer, PRUint32 aByteLength)
 }
 
 nsresult
-nsJSONListener::Consume(const PRUnichar* aBuffer, PRUint32 aByteLength)
+nsJSONListener::Consume(const char16_t* aBuffer, uint32_t aByteLength)
 {
   if (!mBufferedChars.AppendElements(aBuffer, aByteLength))
     return NS_ERROR_FAILURE;
