@@ -1,53 +1,20 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is JPEG Encoding code
- *
- * The Initial Developer of the Original Code is
- * Google Inc.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brett Wilson <brettw@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsJPEGEncoder.h"
-#include "prmem.h"
 #include "prprf.h"
 #include "nsString.h"
 #include "nsStreamUtils.h"
+#include "gfxColor.h"
 
 #include <setjmp.h>
 #include "jerror.h"
 
 using namespace mozilla;
 
-NS_IMPL_THREADSAFE_ISUPPORTS3(nsJPEGEncoder, imgIEncoder, nsIInputStream, nsIAsyncInputStream)
+NS_IMPL_ISUPPORTS(nsJPEGEncoder, imgIEncoder, nsIInputStream, nsIAsyncInputStream)
 
 // used to pass error info through the JPEG library
 struct encoder_error_mgr {
@@ -56,10 +23,10 @@ struct encoder_error_mgr {
 };
 
 nsJPEGEncoder::nsJPEGEncoder() : mFinished(false),
-                                 mImageBuffer(nsnull), mImageBufferSize(0),
+                                 mImageBuffer(nullptr), mImageBufferSize(0),
                                  mImageBufferUsed(0), mImageBufferReadPoint(0),
-                                 mCallback(nsnull),
-                                 mCallbackTarget(nsnull), mNotifyThreshold(0),
+                                 mCallback(nullptr),
+                                 mCallbackTarget(nullptr), mNotifyThreshold(0),
                                  mReentrantMonitor("nsJPEGEncoder.mReentrantMonitor")
 {
 }
@@ -67,8 +34,8 @@ nsJPEGEncoder::nsJPEGEncoder() : mFinished(false),
 nsJPEGEncoder::~nsJPEGEncoder()
 {
   if (mImageBuffer) {
-    PR_Free(mImageBuffer);
-    mImageBuffer = nsnull;
+    moz_free(mImageBuffer);
+    mImageBuffer = nullptr;
   }
 }
 
@@ -80,12 +47,12 @@ nsJPEGEncoder::~nsJPEGEncoder()
 //
 //    Transparency is always discarded.
 
-NS_IMETHODIMP nsJPEGEncoder::InitFromData(const PRUint8* aData,
-                                          PRUint32 aLength, // (unused, req'd by JS)
-                                          PRUint32 aWidth,
-                                          PRUint32 aHeight,
-                                          PRUint32 aStride,
-                                          PRUint32 aInputFormat,
+NS_IMETHODIMP nsJPEGEncoder::InitFromData(const uint8_t* aData,
+                                          uint32_t aLength, // (unused, req'd by JS)
+                                          uint32_t aWidth,
+                                          uint32_t aHeight,
+                                          uint32_t aStride,
+                                          uint32_t aInputFormat,
                                           const nsAString& aOutputOptions)
 {
   NS_ENSURE_ARG(aData);
@@ -107,7 +74,7 @@ NS_IMETHODIMP nsJPEGEncoder::InitFromData(const PRUint8* aData,
   }
 
   // can't initialize more than once
-  if (mImageBuffer != nsnull)
+  if (mImageBuffer != nullptr)
     return NS_ERROR_ALREADY_INITIALIZED;
 
   // options: we only have one option so this is easy
@@ -180,18 +147,18 @@ NS_IMETHODIMP nsJPEGEncoder::InitFromData(const PRUint8* aData,
   // feed it the rows
   if (aInputFormat == INPUT_FORMAT_RGB) {
     while (cinfo.next_scanline < cinfo.image_height) {
-      const PRUint8* row = &aData[cinfo.next_scanline * aStride];
-      jpeg_write_scanlines(&cinfo, const_cast<PRUint8**>(&row), 1);
+      const uint8_t* row = &aData[cinfo.next_scanline * aStride];
+      jpeg_write_scanlines(&cinfo, const_cast<uint8_t**>(&row), 1);
     }
   } else if (aInputFormat == INPUT_FORMAT_RGBA) {
-    PRUint8* row = new PRUint8[aWidth * 3];
+    uint8_t* row = new uint8_t[aWidth * 3];
     while (cinfo.next_scanline < cinfo.image_height) {
-      StripAlpha(&aData[cinfo.next_scanline * aStride], row, aWidth);
+      ConvertRGBARow(&aData[cinfo.next_scanline * aStride], row, aWidth);
       jpeg_write_scanlines(&cinfo, &row, 1);
     }
     delete[] row;
   } else if (aInputFormat == INPUT_FORMAT_HOSTARGB) {
-    PRUint8* row = new PRUint8[aWidth * 3];
+    uint8_t* row = new uint8_t[aWidth * 3];
     while (cinfo.next_scanline < cinfo.image_height) {
       ConvertHostARGBRow(&aData[cinfo.next_scanline * aStride], row, aWidth);
       jpeg_write_scanlines(&cinfo, &row, 1);
@@ -213,16 +180,16 @@ NS_IMETHODIMP nsJPEGEncoder::InitFromData(const PRUint8* aData,
 }
 
 
-NS_IMETHODIMP nsJPEGEncoder::StartImageEncode(PRUint32 aWidth,
-                                              PRUint32 aHeight,
-                                              PRUint32 aInputFormat,
+NS_IMETHODIMP nsJPEGEncoder::StartImageEncode(uint32_t aWidth,
+                                              uint32_t aHeight,
+                                              uint32_t aInputFormat,
                                               const nsAString& aOutputOptions)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 // Returns the number of bytes in the image buffer used.
-NS_IMETHODIMP nsJPEGEncoder::GetImageBufferUsed(PRUint32 *aOutputSize)
+NS_IMETHODIMP nsJPEGEncoder::GetImageBufferUsed(uint32_t *aOutputSize)
 {
   NS_ENSURE_ARG_POINTER(aOutputSize);
   *aOutputSize = mImageBufferUsed;
@@ -237,12 +204,12 @@ NS_IMETHODIMP nsJPEGEncoder::GetImageBuffer(char **aOutputBuffer)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsJPEGEncoder::AddImageFrame(const PRUint8* aData,
-                                           PRUint32 aLength,
-                                           PRUint32 aWidth,
-                                           PRUint32 aHeight,
-                                           PRUint32 aStride,
-                                           PRUint32 aFrameFormat,
+NS_IMETHODIMP nsJPEGEncoder::AddImageFrame(const uint8_t* aData,
+                                           uint32_t aLength,
+                                           uint32_t aWidth,
+                                           uint32_t aHeight,
+                                           uint32_t aStride,
+                                           uint32_t aFrameFormat,
                                            const nsAString& aFrameOptions)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
@@ -257,9 +224,9 @@ NS_IMETHODIMP nsJPEGEncoder::EndImageEncode()
 /* void close (); */
 NS_IMETHODIMP nsJPEGEncoder::Close()
 {
-  if (mImageBuffer != nsnull) {
-    PR_Free(mImageBuffer);
-    mImageBuffer = nsnull;
+  if (mImageBuffer != nullptr) {
+    moz_free(mImageBuffer);
+    mImageBuffer = nullptr;
     mImageBufferSize = 0;
     mImageBufferUsed = 0;
     mImageBufferReadPoint = 0;
@@ -268,7 +235,7 @@ NS_IMETHODIMP nsJPEGEncoder::Close()
 }
 
 /* unsigned long available (); */
-NS_IMETHODIMP nsJPEGEncoder::Available(PRUint32 *_retval)
+NS_IMETHODIMP nsJPEGEncoder::Available(uint64_t *_retval)
 {
   if (!mImageBuffer)
     return NS_BASE_STREAM_CLOSED;
@@ -278,19 +245,19 @@ NS_IMETHODIMP nsJPEGEncoder::Available(PRUint32 *_retval)
 }
 
 /* [noscript] unsigned long read (in charPtr aBuf, in unsigned long aCount); */
-NS_IMETHODIMP nsJPEGEncoder::Read(char * aBuf, PRUint32 aCount,
-                                  PRUint32 *_retval)
+NS_IMETHODIMP nsJPEGEncoder::Read(char * aBuf, uint32_t aCount,
+                                  uint32_t *_retval)
 {
   return ReadSegments(NS_CopySegmentToBuffer, aBuf, aCount, _retval);
 }
 
 /* [noscript] unsigned long readSegments (in nsWriteSegmentFun aWriter, in voidPtr aClosure, in unsigned long aCount); */
-NS_IMETHODIMP nsJPEGEncoder::ReadSegments(nsWriteSegmentFun aWriter, void *aClosure, PRUint32 aCount, PRUint32 *_retval)
+NS_IMETHODIMP nsJPEGEncoder::ReadSegments(nsWriteSegmentFun aWriter, void *aClosure, uint32_t aCount, uint32_t *_retval)
 {
   // Avoid another thread reallocing the buffer underneath us
   ReentrantMonitorAutoEnter autoEnter(mReentrantMonitor);
 
-  PRUint32 maxCount = mImageBufferUsed - mImageBufferReadPoint;
+  uint32_t maxCount = mImageBufferUsed - mImageBufferReadPoint;
   if (maxCount == 0) {
     *_retval = 0;
     return mFinished ? NS_OK : NS_BASE_STREAM_WOULD_BLOCK;
@@ -318,8 +285,8 @@ NS_IMETHODIMP nsJPEGEncoder::IsNonBlocking(bool *_retval)
 }
 
 NS_IMETHODIMP nsJPEGEncoder::AsyncWait(nsIInputStreamCallback *aCallback,
-                                       PRUint32 aFlags,
-                                       PRUint32 aRequestedCount,
+                                       uint32_t aFlags,
+                                       uint32_t aRequestedCount,
                                        nsIEventTarget *aTarget)
 {
   if (aFlags != 0)
@@ -350,46 +317,48 @@ NS_IMETHODIMP nsJPEGEncoder::CloseWithStatus(nsresult aStatus)
   return Close();
 }
 
+
+
 // nsJPEGEncoder::ConvertHostARGBRow
 //
 //    Our colors are stored with premultiplied alphas, but we need
 //    an output with no alpha in machine-independent byte order.
 //
 //    See gfx/cairo/cairo/src/cairo-png.c
-
 void
-nsJPEGEncoder::ConvertHostARGBRow(const PRUint8* aSrc, PRUint8* aDest,
-                                 PRUint32 aPixelWidth)
+nsJPEGEncoder::ConvertHostARGBRow(const uint8_t* aSrc, uint8_t* aDest,
+                                  uint32_t aPixelWidth)
 {
-  for (PRUint32 x = 0; x < aPixelWidth; x ++) {
-    const PRUint32& pixelIn = ((const PRUint32*)(aSrc))[x];
-    PRUint8 *pixelOut = &aDest[x * 3];
+  for (uint32_t x = 0; x < aPixelWidth; x++) {
+    const uint32_t& pixelIn = ((const uint32_t*)(aSrc))[x];
+    uint8_t *pixelOut = &aDest[x * 3];
 
-    pixelOut[0] = (((pixelIn & 0xff0000) >> 16));
-    pixelOut[1] = (((pixelIn & 0x00ff00) >>  8));
-    pixelOut[2] = (((pixelIn & 0x0000ff) >>  0));
+    pixelOut[0] = (pixelIn & 0xff0000) >> 16;
+    pixelOut[1] = (pixelIn & 0x00ff00) >>  8;
+    pixelOut[2] = (pixelIn & 0x0000ff) >>  0;
   }
 }
 
-
-// nsJPEGEncoder::StripAlpha
-//
-//    Input is RGBA, output is RGB
-
+/**
+ * nsJPEGEncoder::ConvertRGBARow
+ *
+ * Input is RGBA, output is RGB, so we should alpha-premultiply.
+ */
 void
-nsJPEGEncoder::StripAlpha(const PRUint8* aSrc, PRUint8* aDest,
-                          PRUint32 aPixelWidth)
+nsJPEGEncoder::ConvertRGBARow(const uint8_t* aSrc, uint8_t* aDest,
+                              uint32_t aPixelWidth)
 {
-  for (PRUint32 x = 0; x < aPixelWidth; x ++) {
-    const PRUint8* pixelIn = &aSrc[x * 4];
-    PRUint8* pixelOut = &aDest[x * 3];
-    pixelOut[0] = pixelIn[0];
-    pixelOut[1] = pixelIn[1];
-    pixelOut[2] = pixelIn[2];
+  for (uint32_t x = 0; x < aPixelWidth; x++) {
+    const uint8_t* pixelIn = &aSrc[x * 4];
+    uint8_t* pixelOut = &aDest[x * 3];
+
+    uint8_t alpha = pixelIn[3];
+    pixelOut[0] = gfxPreMultiply(pixelIn[0], alpha);
+    pixelOut[1] = gfxPreMultiply(pixelIn[1], alpha);
+    pixelOut[2] = gfxPreMultiply(pixelIn[2], alpha);
   }
 }
 
-
 // nsJPEGEncoder::initDestination
 //
 //    Initialize destination. This is called by jpeg_start_compress() before
@@ -403,7 +372,7 @@ nsJPEGEncoder::initDestination(jpeg_compress_struct* cinfo)
   NS_ASSERTION(! that->mImageBuffer, "Image buffer already initialized");
 
   that->mImageBufferSize = 8192;
-  that->mImageBuffer = (PRUint8*)PR_Malloc(that->mImageBufferSize);
+  that->mImageBuffer = (uint8_t*)moz_malloc(that->mImageBufferSize);
   that->mImageBufferUsed = 0;
 
   cinfo->dest->next_output_byte = that->mImageBuffer;
@@ -437,18 +406,20 @@ nsJPEGEncoder::emptyOutputBuffer(jpeg_compress_struct* cinfo)
   // expand buffer, just double size each time
   that->mImageBufferSize *= 2;
 
-  PRUint8* newBuf = (PRUint8*)PR_Realloc(that->mImageBuffer,
-                                         that->mImageBufferSize);
+  uint8_t* newBuf = (uint8_t*)moz_realloc(that->mImageBuffer,
+                                          that->mImageBufferSize);
   if (! newBuf) {
     // can't resize, just zero (this will keep us from writing more)
-    PR_Free(that->mImageBuffer);
-    that->mImageBuffer = nsnull;
+    moz_free(that->mImageBuffer);
+    that->mImageBuffer = nullptr;
     that->mImageBufferSize = 0;
     that->mImageBufferUsed = 0;
 
-    // this seems to be the only way to do errors through the JPEG library
+    // This seems to be the only way to do errors through the JPEG library.  We
+    // pass an nsresult masquerading as an int, which works because the
+    // setjmp() caller casts it back.
     longjmp(((encoder_error_mgr*)(cinfo->err))->setjmp_buffer,
-            NS_ERROR_OUT_OF_MEMORY);
+            static_cast<int>(NS_ERROR_OUT_OF_MEMORY));
   }
   that->mImageBuffer = newBuf;
 
@@ -498,8 +469,9 @@ nsJPEGEncoder::errorExit(jpeg_common_struct* cinfo)
       error_code = NS_ERROR_FAILURE;
   }
 
-  // Return control to the setjmp point.
-  longjmp(err->setjmp_buffer, error_code);
+  // Return control to the setjmp point.  We pass an nsresult masquerading as
+  // an int, which works because the setjmp() caller casts it back.
+  longjmp(err->setjmp_buffer, static_cast<int>(error_code));
 }
 
 void
@@ -516,9 +488,7 @@ nsJPEGEncoder::NotifyListener()
        mFinished)) {
     nsCOMPtr<nsIInputStreamCallback> callback;
     if (mCallbackTarget) {
-      NS_NewInputStreamReadyEvent(getter_AddRefs(callback),
-                                  mCallback,
-                                  mCallbackTarget);
+      callback = NS_NewInputStreamReadyEvent(mCallback, mCallbackTarget);
     } else {
       callback = mCallback;
     }
@@ -526,8 +496,8 @@ nsJPEGEncoder::NotifyListener()
     NS_ASSERTION(callback, "Shouldn't fail to make the callback");
     // Null the callback first because OnInputStreamReady could reenter
     // AsyncWait
-    mCallback = nsnull;
-    mCallbackTarget = nsnull;
+    mCallback = nullptr;
+    mCallbackTarget = nullptr;
     mNotifyThreshold = 0;
 
     callback->OnInputStreamReady(this);
