@@ -3,7 +3,7 @@
 
 load(libdir + "asserts.js");
 
-var g = newGlobal('new-compartment');
+var g = newGlobal();
 g.debuggeeGlobal = this;
 g.eval("(" + function () {
         var dbg = new Debugger(debuggeeGlobal);
@@ -18,5 +18,5 @@ g.eval("(" + function () {
     } + ")();");
 
 function f() { debugger; }
-for (var i = 0; i < HOTLOOP + 2; i++)
+for (var i = 0; i < 10; i++)
     assertThrowsValue(f, i);
