@@ -1,57 +1,99 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=99:
- *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is SpiderMonkey JSON.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jeff Walden <jwalden+code@mit.edu> (original author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "jsarray.h"
-#include "jsnum.h"
 #include "jsonparser.h"
 
+#include "mozilla/RangedPtr.h"
+
+#include <ctype.h>
+
+#include "jsarray.h"
+#include "jscompartment.h"
+#include "jsnum.h"
+#include "jsprf.h"
+
+#include "vm/StringBuffer.h"
+
 #include "jsobjinlines.h"
-#include "jsstrinlines.h"
 
 using namespace js;
 
+using mozilla::RangedPtr;
+
+JSONParser::~JSONParser()
+{
+    for (size_t i = 0; i < stack.length(); i++) {
+        if (stack[i].state == FinishArrayElement)
+            js_delete(&stack[i].elements());
+        else
+            js_delete(&stack[i].properties());
+    }
+
+    for (size_t i = 0; i < freeElements.length(); i++)
+        js_delete(freeElements[i]);
+
+    for (size_t i = 0; i < freeProperties.length(); i++)
+        js_delete(freeProperties[i]);
+}
+
+void
+JSONParser::trace(JSTracer *trc)
+{
+    for (size_t i = 0; i < stack.length(); i++) {
+        if (stack[i].state == FinishArrayElement) {
+            ElementVector &elements = stack[i].elements();
+            for (size_t j = 0; j < elements.length(); j++)
+                gc::MarkValueRoot(trc, &elements[j], "JSONParser element");
+        } else {
+            PropertyVector &properties = stack[i].properties();
+            for (size_t j = 0; j < properties.length(); j++) {
+                gc::MarkValueRoot(trc, &properties[j].value, "JSONParser property value");
+                gc::MarkIdRoot(trc, &properties[j].id, "JSONParser property id");
+            }
+        }
+    }
+}
+
+void
+JSONParser::getTextPosition(uint32_t *column, uint32_t *line)
+{
+    ConstTwoByteChars ptr = begin;
+    uint32_t col = 1;
+    uint32_t row = 1;
+    for (; ptr < current; ptr++) {
+        if (*ptr == '\n' || *ptr == '\r') {
+            ++row;
+            col = 1;
+            // \r\n is treated as a single newline.
+            if (ptr + 1 < current && *ptr == '\r' && *(ptr + 1) == '\n')
+                ++ptr;
+        } else {
+            ++col;
+        }
+    }
+    *column = col;
+    *line = row;
+}
+
 void
 JSONParser::error(const char *msg)
 {
-    if (errorHandling == RaiseError)
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_JSON_BAD_PARSE, msg);
+    if (errorHandling == RaiseError) {
+        uint32_t column = 1, line = 1;
+        getTextPosition(&column, &line);
+
+        const size_t MaxWidth = sizeof("4294967295");
+        char columnNumber[MaxWidth];
+        JS_snprintf(columnNumber, sizeof columnNumber, "%lu", column);
+        char lineNumber[MaxWidth];
+        JS_snprintf(lineNumber, sizeof lineNumber, "%lu", line);
+
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_JSON_BAD_PARSE,
+                             msg, lineNumber, columnNumber);
+    }
 }
 
 bool
@@ -81,14 +123,14 @@ JSONParser::readString()
      * Optimization: if the source contains no escaped characters, create the
      * string directly from the source text.
      */
-    RangeCheckedPointer<const jschar> start = current;
+    RangedPtr<const jschar> start = current;
     for (; current < end; current++) {
         if (*current == '"') {
             size_t length = current - start;
             current++;
             JSFlatString *str = (ST == JSONParser::PropertyName)
-                                ? js_AtomizeChars(cx, start, length)
-                                : js_NewStringCopyN(cx, start, length);
+                                ? AtomizeChars(cx, start.get(), length)
+                                : js_NewStringCopyN<CanGC>(cx, start.get(), length);
             if (!str)
                 return token(OOM);
             return stringToken(str);
@@ -110,7 +152,7 @@ JSONParser::readString()
      */
     StringBuffer buffer(cx);
     do {
-        if (start < current && !buffer.append(start, current))
+        if (start < current && !buffer.append(start.get(), current.get()))
             return token(OOM);
 
         if (current >= end)
@@ -127,6 +169,7 @@ JSONParser::readString()
         }
 
         if (c != '\\') {
+            --current;
             error("bad character in string literal");
             return token(Error);
         }
@@ -145,25 +188,37 @@ JSONParser::readString()
           case 't':  c = '\t'; break;
 
           case 'u':
-            if (end - current < 4) {
+            if (end - current < 4 ||
+                !(JS7_ISHEX(current[0]) &&
+                  JS7_ISHEX(current[1]) &&
+                  JS7_ISHEX(current[2]) &&
+                  JS7_ISHEX(current[3])))
+            {
+                // Point to the first non-hexadecimal character (which may be
+                // missing).
+                if (current == end || !JS7_ISHEX(current[0]))
+                    ; // already at correct location
+                else if (current + 1 == end || !JS7_ISHEX(current[1]))
+                    current += 1;
+                else if (current + 2 == end || !JS7_ISHEX(current[2]))
+                    current += 2;
+                else if (current + 3 == end || !JS7_ISHEX(current[3]))
+                    current += 3;
+                else
+                    MOZ_ASSUME_UNREACHABLE("logic error determining first erroneous character");
+
                 error("bad Unicode escape");
                 return token(Error);
             }
-            if (JS7_ISHEX(current[0]) &&
-                JS7_ISHEX(current[1]) &&
-                JS7_ISHEX(current[2]) &&
-                JS7_ISHEX(current[3]))
-            {
-                c = (JS7_UNHEX(current[0]) << 12)
-                  | (JS7_UNHEX(current[1]) << 8)
-                  | (JS7_UNHEX(current[2]) << 4)
-                  | (JS7_UNHEX(current[3]));
-                current += 4;
-                break;
-            }
-            /* FALL THROUGH */
+            c = (JS7_UNHEX(current[0]) << 12)
+              | (JS7_UNHEX(current[1]) << 8)
+              | (JS7_UNHEX(current[2]) << 4)
+              | (JS7_UNHEX(current[3]));
+            current += 4;
+            break;
 
           default:
+            current--;
             error("bad escaped character");
             return token(Error);
         }
@@ -200,7 +255,7 @@ JSONParser::readNumber()
         return token(Error);
     }
 
-    const RangeCheckedPointer<const jschar> digitStart = current;
+    const RangedPtr<const jschar> digitStart = current;
 
     /* 0|[1-9][0-9]+ */
     if (!JS7_ISDEC(*current)) {
@@ -216,9 +271,19 @@ JSONParser::readNumber()
 
     /* Fast path: no fractional or exponent part. */
     if (current == end || (*current != '.' && *current != 'e' && *current != 'E')) {
+        TwoByteChars chars(digitStart.get(), current - digitStart);
+        if (chars.length() < strlen("9007199254740992")) {
+            // If the decimal number is shorter than the length of 2**53, (the
+            // largest number a double can represent with integral precision),
+            // parse it using a decimal-only parser.  This comparison is
+            // conservative but faster than a fully-precise check.
+            double d = ParseDecimalNumber(chars);
+            return numberToken(negative ? -d : d);
+        }
+
+        double d;
         const jschar *dummy;
-        jsdouble d;
-        if (!GetPrefixInteger(cx, digitStart, current, 10, &dummy, &d))
+        if (!GetPrefixInteger(cx, digitStart.get(), current.get(), 10, &dummy, &d))
             return token(OOM);
         JS_ASSERT(current == dummy);
         return numberToken(negative ? -d : d);
@@ -262,9 +327,9 @@ JSONParser::readNumber()
         }
     }
 
-    jsdouble d;
+    double d;
     const jschar *finish;
-    if (!js_strtod(cx, digitStart, current, &finish, &d))
+    if (!js_strtod(cx, digitStart.get(), current.get(), &finish, &d))
         return token(OOM);
     JS_ASSERT(current == finish);
     return numberToken(negative ? -d : d);
@@ -382,7 +447,7 @@ JSONParser::advanceAfterObjectOpen()
 }
 
 static inline void
-AssertPastValue(const RangeCheckedPointer<const jschar> current)
+AssertPastValue(const RangedPtr<const jschar> current)
 {
     /*
      * We're past an arbitrary JSON value, so the previous character is
@@ -449,19 +514,6 @@ JSONParser::advancePropertyName()
     if (*current == '"')
         return readString<PropertyName>();
 
-    if (parsingMode == LegacyJSON && *current == '}') {
-        /*
-         * Previous JSON parsing accepted trailing commas in non-empty object
-         * syntax, and some users depend on this.  (Specifically, Places data
-         * serialization in versions of Firefox before 4.0.  We can remove this
-         * mode when profile upgrades from 3.6 become unsupported.)  Permit
-         * such trailing commas only when legacy parsing is specifically
-         * requested.
-         */
-        current++;
-        return token(ObjectClose);
-    }
-
     error("expected double-quoted property name");
     return token(Error);
 }
@@ -513,40 +565,108 @@ JSONParser::advanceAfterProperty()
     return token(Error);
 }
 
-/*
- * This enum is local to JSONParser::parse, below, but ISO C++98 doesn't allow
- * templates to depend on local types.  Boo-urns!
- */
-enum ParserState { FinishArrayElement, FinishObjectMember, JSONValue };
+JSObject *
+JSONParser::createFinishedObject(PropertyVector &properties)
+{
+    /*
+     * Look for an existing cached type and shape for objects with this set of
+     * properties.
+     */
+    {
+        JSObject *obj = cx->compartment()->types.newTypedObject(cx, properties.begin(),
+                                                                properties.length());
+        if (obj)
+            return obj;
+    }
+
+    /*
+     * Make a new object sized for the given number of properties and fill its
+     * shape in manually.
+     */
+    gc::AllocKind allocKind = gc::GetGCObjectKind(properties.length());
+    RootedObject obj(cx, NewBuiltinClassInstance(cx, &JSObject::class_, allocKind));
+    if (!obj)
+        return nullptr;
+
+    RootedId propid(cx);
+    RootedValue value(cx);
+
+    for (size_t i = 0; i < properties.length(); i++) {
+        propid = properties[i].id;
+        value = properties[i].value;
+        if (!DefineNativeProperty(cx, obj, propid, value, JS_PropertyStub, JS_StrictPropertyStub,
+                                  JSPROP_ENUMERATE)) {
+            return nullptr;
+        }
+    }
+
+    /*
+     * Try to assign a new type to the object with type information for its
+     * properties, and update the initializer type object cache with this
+     * object's final shape.
+     */
+    cx->compartment()->types.fixObjectType(cx, obj);
+
+    return obj;
+}
+
+inline bool
+JSONParser::finishObject(MutableHandleValue vp, PropertyVector &properties)
+{
+    JS_ASSERT(&properties == &stack.back().properties());
+
+    JSObject *obj = createFinishedObject(properties);
+    if (!obj)
+        return false;
+
+    vp.setObject(*obj);
+    if (!freeProperties.append(&properties))
+        return false;
+    stack.popBack();
+    return true;
+}
+
+inline bool
+JSONParser::finishArray(MutableHandleValue vp, ElementVector &elements)
+{
+    JS_ASSERT(&elements == &stack.back().elements());
+
+    JSObject *obj = NewDenseCopiedArray(cx, elements.length(), elements.begin());
+    if (!obj)
+        return false;
+
+    /* Try to assign a new type to the array according to its elements. */
+    cx->compartment()->types.fixArrayType(cx, obj);
+
+    vp.setObject(*obj);
+    if (!freeElements.append(&elements))
+        return false;
+    stack.popBack();
+    return true;
+}
 
 bool
-JSONParser::parse(Value *vp)
+JSONParser::parse(MutableHandleValue vp)
 {
-    Vector<ParserState> stateStack(cx);
-    AutoValueVector valueStack(cx);
+    RootedValue value(cx);
+    JS_ASSERT(stack.empty());
 
-    *vp = UndefinedValue();
+    vp.setUndefined();
 
     Token token;
     ParserState state = JSONValue;
     while (true) {
         switch (state) {
           case FinishObjectMember: {
-            Value v = valueStack.popCopy();
-            /*
-             * NB: Relies on js_DefineNativeProperty performing
-             *     js_CheckForStringIndex.
-             */
-            jsid propid = ATOM_TO_JSID(&valueStack.popCopy().toString()->asAtom());
-            if (!DefineNativeProperty(cx, &valueStack.back().toObject(), propid, v,
-                                      PropertyStub, StrictPropertyStub, JSPROP_ENUMERATE,
-                                      0, 0))
-            {
-                return false;
-            }
+            PropertyVector &properties = stack.back().properties();
+            properties.back().value = value;
+
             token = advanceAfterProperty();
-            if (token == ObjectClose)
+            if (token == ObjectClose) {
+                if (!finishObject(&value, properties))
+                    return false;
                 break;
+            }
             if (token != Comma) {
                 if (token == OOM)
                     return false;
@@ -560,22 +680,17 @@ JSONParser::parse(Value *vp)
 
           JSONMember:
             if (token == String) {
-                if (!valueStack.append(atomValue()))
+                jsid id = AtomToId(atomValue());
+                PropertyVector &properties = stack.back().properties();
+                if (!properties.append(IdValuePair(id)))
                     return false;
                 token = advancePropertyColon();
                 if (token != Colon) {
                     JS_ASSERT(token == Error);
                     return errorReturn();
                 }
-                if (!stateStack.append(FinishObjectMember))
-                    return false;
                 goto JSONValue;
             }
-            if (token == ObjectClose) {
-                JS_ASSERT(state == FinishObjectMember);
-                JS_ASSERT(parsingMode == LegacyJSON);
-                break;
-            }
             if (token == OOM)
                 return false;
             if (token != Error)
@@ -583,17 +698,17 @@ JSONParser::parse(Value *vp)
             return errorReturn();
 
           case FinishArrayElement: {
-            Value v = valueStack.popCopy();
-            if (!js_ArrayCompPush(cx, &valueStack.back().toObject(), v))
+            ElementVector &elements = stack.back().elements();
+            if (!elements.append(value.get()))
                 return false;
             token = advanceAfterArrayElement();
-            if (token == Comma) {
-                if (!stateStack.append(FinishArrayElement))
-                    return false;
+            if (token == Comma)
                 goto JSONValue;
-            }
-            if (token == ArrayClose)
+            if (token == ArrayClose) {
+                if (!finishArray(&value, elements))
+                    return false;
                 break;
+            }
             JS_ASSERT(token == Error);
             return errorReturn();
           }
@@ -604,66 +719,72 @@ JSONParser::parse(Value *vp)
           JSONValueSwitch:
             switch (token) {
               case String:
+                value = stringValue();
+                break;
               case Number:
-                if (!valueStack.append(token == String ? stringValue() : numberValue()))
-                    return false;
+                value = numberValue();
                 break;
               case True:
-                if (!valueStack.append(BooleanValue(true)))
-                    return false;
+                value = BooleanValue(true);
                 break;
               case False:
-                if (!valueStack.append(BooleanValue(false)))
-                    return false;
+                value = BooleanValue(false);
                 break;
               case Null:
-                if (!valueStack.append(NullValue()))
-                    return false;
+                value = NullValue();
                 break;
 
               case ArrayOpen: {
-                JSObject *obj = NewDenseEmptyArray(cx);
-                if (!obj || !valueStack.append(ObjectValue(*obj)))
+                ElementVector *elements;
+                if (!freeElements.empty()) {
+                    elements = freeElements.popCopy();
+                    elements->clear();
+                } else {
+                    elements = cx->new_<ElementVector>(cx);
+                    if (!elements)
+                        return false;
+                }
+                if (!stack.append(elements))
                     return false;
+
                 token = advance();
-                if (token == ArrayClose)
+                if (token == ArrayClose) {
+                    if (!finishArray(&value, *elements))
+                        return false;
                     break;
-                if (!stateStack.append(FinishArrayElement))
-                    return false;
+                }
                 goto JSONValueSwitch;
               }
 
               case ObjectOpen: {
-                JSObject *obj = NewBuiltinClassInstance(cx, &js_ObjectClass);
-                if (!obj || !valueStack.append(ObjectValue(*obj)))
+                PropertyVector *properties;
+                if (!freeProperties.empty()) {
+                    properties = freeProperties.popCopy();
+                    properties->clear();
+                } else {
+                    properties = cx->new_<PropertyVector>(cx);
+                    if (!properties)
+                        return false;
+                }
+                if (!stack.append(properties))
                     return false;
+
                 token = advanceAfterObjectOpen();
-                if (token == ObjectClose)
+                if (token == ObjectClose) {
+                    if (!finishObject(&value, *properties))
+                        return false;
                     break;
+                }
                 goto JSONMember;
               }
 
               case ArrayClose:
-                if (parsingMode == LegacyJSON &&
-                    !stateStack.empty() &&
-                    stateStack.back() == FinishArrayElement) {
-                    /*
-                     * Previous JSON parsing accepted trailing commas in
-                     * non-empty array syntax, and some users depend on this.
-                     * (Specifically, Places data serialization in versions of
-                     * Firefox prior to 4.0.  We can remove this mode when
-                     * profile upgrades from 3.6 become unsupported.)  Permit
-                     * such trailing commas only when specifically
-                     * instructed to do so.
-                     */
-                    stateStack.popBack();
-                    break;
-                }
-                /* FALL THROUGH */
-
               case ObjectClose:
               case Colon:
               case Comma:
+                // Move the current pointer backwards so that the position
+                // reported in the error message is correct.
+                --current;
                 error("unexpected character");
                 return errorReturn();
 
@@ -676,9 +797,9 @@ JSONParser::parse(Value *vp)
             break;
         }
 
-        if (stateStack.empty())
+        if (stack.empty())
             break;
-        state = stateStack.popCopy();
+        state = stack.back().state;
     }
 
     for (; current < end; current++) {
@@ -689,7 +810,8 @@ JSONParser::parse(Value *vp)
     }
 
     JS_ASSERT(end == current);
-    JS_ASSERT(valueStack.length() == 1);
-    *vp = valueStack[0];
+    JS_ASSERT(stack.empty());
+
+    vp.set(value);
     return true;
 }
