@@ -84,8 +84,8 @@ var tabs1 = [
     title: "Mozilla",
     profile: "profile1"
   },
-  { uri: "data:text/html,<html><head><title>Firefox</title></head><body>Firefox</body></html>",
-    title: "Firefox",
+  { uri: "data:text/html,<html><head><title>IceCat</title></head><body>IceCat</body></html>",
+    title: "IceCat",
     profile: "profile1"
   },
   { uri: "data:text/html,<html><head><title>Weave</title></head><body>Weave</body></html>",
