@@ -1,45 +1,16 @@
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is cookie manager code.
-#
-# The Initial Developer of the Original Code is
-# Michiel van Leeuwen.
-# Portions created by the Initial Developer are Copyright (C) 2002
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Ehsan Akhgari <ehsan.akhgari@gmail.com>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+// -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const nsICookieAcceptDialog = Components.interfaces.nsICookieAcceptDialog;
 const nsIDialogParamBlock = Components.interfaces.nsIDialogParamBlock;
 const nsICookie = Components.interfaces.nsICookie;
 const nsICookiePromptService = Components.interfaces.nsICookiePromptService;
 
+Components.utils.import("resource://gre/modules/PrivateBrowsingUtils.jsm");
+
 var params; 
 var cookieBundle;
 var gDateService = null;
@@ -55,7 +26,7 @@ function onload()
   var dialog = document.documentElement;
 
   document.getElementById("Button2").collapsed = false;
-  
+
   document.getElementById("ok").label = dialog.getAttribute("acceptLabel");
   document.getElementById("ok").accessKey = dialog.getAttribute("acceptKey");
   document.getElementById("Button2").label = dialog.getAttribute("extra1Label");
@@ -98,7 +69,6 @@ function onload()
       params = window.arguments[0].QueryInterface(nsIDialogParamBlock);
       var objects = params.objects;
       var cookie = params.objects.queryElementAt(0,nsICookie);
-      
       var cookiesFromHost = params.GetInt(nsICookieAcceptDialog.COOKIESFROMHOST);
 
       var messageFormat;
@@ -161,9 +131,7 @@ function onload()
 
   // The Private Browsing service might not be available
   try {
-    var pb = Components.classes["@mozilla.org/privatebrowsing;1"].
-             getService(Components.interfaces.nsIPrivateBrowsingService);
-    if (pb.privateBrowsingEnabled) {
+    if (window.opener && PrivateBrowsingUtils.isWindowPrivate(window.opener)) {
       var persistCheckbox = document.getElementById("persistDomainAcceptance");
       persistCheckbox.removeAttribute("checked");
       persistCheckbox.setAttribute("disabled", "true");
