@@ -1,47 +1,13 @@
-# -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is mozilla.org configuration viewer.
-#
-# The Initial Developer of the Original Code is
-# Netscape Communications Corporation.
-# Portions created by the Initial Developer are Copyright (C) 2002
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Chip Clark <chipc@netscape.com>
-#   Seth Spitzer <sspitzer@netscape.com>
-#   Neil Rashbrook <neil@parkwaycc.co.uk>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+// -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 const nsIPrefLocalizedString = Components.interfaces.nsIPrefLocalizedString;
 const nsISupportsString = Components.interfaces.nsISupportsString;
-const nsIPromptService = Components.interfaces.nsIPromptService;
-const nsIPrefService = Components.interfaces.nsIPrefService;
 const nsIPrefBranch = Components.interfaces.nsIPrefBranch;
 const nsIClipboardHelper = Components.interfaces.nsIClipboardHelper;
 const nsIAtomService = Components.interfaces.nsIAtomService;
@@ -52,13 +18,11 @@ const nsPrefService_CONTRACTID = "@mozilla.org/preferences-service;1";
 const nsClipboardHelper_CONTRACTID = "@mozilla.org/widget/clipboardhelper;1";
 const nsAtomService_CONTRACTID = "@mozilla.org/atom-service;1";
 
-const gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
-const gPrefService = Components.classes[nsPrefService_CONTRACTID].getService(nsIPrefService);
-const gPrefBranch = gPrefService.getBranch(null).QueryInterface(Components.interfaces.nsIPrefBranch2);
+const gPrefBranch = Services.prefs;
 const gClipboardHelper = Components.classes[nsClipboardHelper_CONTRACTID].getService(nsIClipboardHelper);
 const gAtomService = Components.classes[nsAtomService_CONTRACTID].getService(nsIAtomService);
 
-var gLockAtoms = [gAtomService.getAtom("default"), gAtomService.getAtom("user"), gAtomService.getAtom("locked")];
+var gLockProps = ["default", "user", "locked"];
 // we get these from a string bundle
 var gLockStrs = [];
 var gTypeStrs = [];
@@ -93,12 +57,14 @@ var view = {
         return value;
     }
   },
-  getRowProperties : function(index, prop) {},
-  getCellProperties : function(index, col, prop) {
+  getRowProperties : function(index) { return ""; },
+  getCellProperties : function(index, col) {
     if (index in gPrefView)
-      prop.AppendElement(gLockAtoms[gPrefView[index].lockCol]);
+      return gLockProps[gPrefView[index].lockCol];
+
+    return "";
   },
-  getColumnProperties : function(col, prop) {},
+  getColumnProperties : function(col) { return ""; },
   treebox : null,
   selection : null,
   isContainer : function(index) { return false; },
@@ -229,9 +195,6 @@ var gPrefListener =
     if (topic != "nsPref:changed")
       return;
 
-    if (/^capability\./.test(prefName)) // avoid displaying "private" preferences
-      return;
-
     var arrayIndex = gPrefArray.length;
     var viewIndex = arrayIndex;
     var selectedIndex = view.selection.currentIndex;
@@ -382,14 +345,7 @@ function onConfigLoad()
 // Unhide the warning message
 function ShowPrefs()
 {
-  var prefArray = gPrefBranch.getChildList("");
-
-  prefArray.forEach(function (prefName) {
-    if (/^capability\./.test(prefName)) // avoid displaying "private" preferences
-      return;
-
-    fetchPref(prefName, gPrefArray.length);
-  });
+  gPrefBranch.getChildList("").forEach(fetchPref);
 
   var descending = document.getElementsByAttribute("sortDirection", "descending");
   if (descending.item(0)) {
@@ -444,6 +400,10 @@ function onConfigUnload()
 
 function FilterPrefs()
 {
+  if (document.getElementById("configDeck").getAttribute("selectedIndex") != 1) {
+    return;
+  }
+
   var substring = document.getElementById("textbox").value;
   // Check for "/regex/[i]"
   if (substring.charAt(0) == '/') {
@@ -575,17 +535,17 @@ function updateContextMenu()
 function copyPref()
 {
   var pref = gPrefView[view.selection.currentIndex];
-  gClipboardHelper.copyString(pref.prefCol + ';' + pref.valueCol);
+  gClipboardHelper.copyString(pref.prefCol + ';' + pref.valueCol, document);
 }
 
 function copyName()
 {
-  gClipboardHelper.copyString(gPrefView[view.selection.currentIndex].prefCol);
+  gClipboardHelper.copyString(gPrefView[view.selection.currentIndex].prefCol, document);
 }
 
 function copyValue()
 {
-  gClipboardHelper.copyString(gPrefView[view.selection.currentIndex].valueCol);
+  gClipboardHelper.copyString(gPrefView[view.selection.currentIndex].valueCol, document);
 }
 
 function ModifySelected()
@@ -604,12 +564,13 @@ function NewPref(type)
 {
   var result = { value: "" };
   var dummy = { value: 0 };
-  if (gPromptService.prompt(window,
-                            gConfigBundle.getFormattedString("new_title", [gTypeStrs[type]]),
-                            gConfigBundle.getString("new_prompt"),
-                            result,
-                            null,
-                            dummy)) {
+  if (Services.prompt.prompt(window,
+                             gConfigBundle.getFormattedString("new_title",
+                                                              [gTypeStrs[type]]),
+                             gConfigBundle.getString("new_prompt"),
+                             result,
+                             null,
+                             dummy)) {
     result.value = result.value.trim();
     if (!result.value) {
       return;
@@ -645,13 +606,13 @@ function ModifyPref(entry)
   var title = gConfigBundle.getFormattedString("modify_title", [gTypeStrs[entry.typeCol]]);
   if (entry.typeCol == nsIPrefBranch.PREF_BOOL) {
     var check = { value: entry.valueCol == "false" };
-    if (!entry.valueCol && !gPromptService.select(window, title, entry.prefCol, 2, [false, true], check))
+    if (!entry.valueCol && !Services.prompt.select(window, title, entry.prefCol, 2, [false, true], check))
       return false;
     gPrefBranch.setBoolPref(entry.prefCol, check.value);
   } else {
     var result = { value: entry.valueCol };
     var dummy = { value: 0 };
-    if (!gPromptService.prompt(window, title, entry.prefCol, result, null, dummy))
+    if (!Services.prompt.prompt(window, title, entry.prefCol, result, null, dummy))
       return false;
     if (entry.typeCol == nsIPrefBranch.PREF_INT) {
       // | 0 converts to integer or 0; - 0 to float or NaN.
@@ -660,7 +621,7 @@ function ModifyPref(entry)
       if (val != result.value - 0) {
         var err_title = gConfigBundle.getString("nan_title");
         var err_text = gConfigBundle.getString("nan_text");
-        gPromptService.alert(window, err_title, err_text);
+        Services.prompt.alert(window, err_title, err_text);
         return false;
       }
       gPrefBranch.setIntPref(entry.prefCol, val);
@@ -671,6 +632,6 @@ function ModifyPref(entry)
     }
   }
 
-  gPrefService.savePrefFile(null);
+  Services.prefs.savePrefFile(null);
   return true;
 }
