@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is C++ hashtable templates.
- *
- * The Initial Developer of the Original Code is
- * Benjamin Smedberg.
- * Portions created by the Initial Developer are Copyright (C) 2002
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsTHashtable_h__
 #define nsTHashtable_h__
@@ -41,13 +9,19 @@
 #include "nscore.h"
 #include "pldhash.h"
 #include "nsDebug.h"
-#include NEW_H
+#include "mozilla/MemoryChecking.h"
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/Move.h"
+#include "mozilla/fallible.h"
+#include "mozilla/PodOperations.h"
+
+#include <new>
 
 // helper function for nsTHashtable::Clear()
 NS_COM_GLUE PLDHashOperator
 PL_DHashStubEnumRemove(PLDHashTable    *table,
                        PLDHashEntryHdr *entry,
-                       PRUint32         ordinal,
+                       uint32_t         ordinal,
                        void            *userArg);
 
 
@@ -66,7 +40,7 @@ PL_DHashStubEnumRemove(PLDHashTable    *table,
  *   {
  *   public: or friend nsTHashtable<EntryType>;
  *     // KeyType is what we use when Get()ing or Put()ing this entry
- *     // this should either be a simple datatype (PRUint32, nsISupports*) or
+ *     // this should either be a simple datatype (uint32_t, nsISupports*) or
  *     // a const reference (const nsAString&)
  *     typedef something KeyType;
  *     // KeyTypePointer is the pointer-version of KeyType, because pldhash.h
@@ -75,9 +49,10 @@ PL_DHashStubEnumRemove(PLDHashTable    *table,
  *
  *     EntryType(KeyTypePointer aKey);
  *
- *     // the copy constructor must be defined, even if AllowMemMove() == true
- *     // or you will cause link errors!
- *     EntryType(const EntryType& aEnt);
+ *     // A copy or C++11 Move constructor must be defined, even if
+ *     // AllowMemMove() == true, otherwise you will cause link errors.
+ *     EntryType(const EntryType& aEnt);  // Either this...
+ *     EntryType(EntryType&& aEnt);       // ...or this
  *
  *     // the destructor must be defined... or you will cause link errors!
  *     ~EntryType();
@@ -93,7 +68,7 @@ PL_DHashStubEnumRemove(PLDHashTable    *table,
  *
  *     // ALLOW_MEMMOVE can we move this class with memmove(), or do we have
  *     // to use the copy constructor?
- *     enum { ALLOW_MEMMOVE = PR_(TRUE or FALSE) };
+ *     enum { ALLOW_MEMMOVE = true/false };
  *   }</pre>
  *
  * @see nsInterfaceHashtable
@@ -105,36 +80,32 @@ PL_DHashStubEnumRemove(PLDHashTable    *table,
 template<class EntryType>
 class nsTHashtable
 {
+  typedef mozilla::fallible_t fallible_t;
+
 public:
-  /**
-   * A dummy constructor; you must call Init() before using this class.
-   */
-  nsTHashtable();
+  // Separate constructors instead of default aInitSize parameter since
+  // otherwise the default no-arg constructor isn't found.
+  nsTHashtable()
+  {
+    Init(PL_DHASH_MIN_SIZE);
+  }
+  explicit nsTHashtable(uint32_t aInitSize)
+  {
+    Init(aInitSize);
+  }
 
   /**
    * destructor, cleans up and deallocates
    */
   ~nsTHashtable();
 
-  /**
-   * Initialize the table.  This function must be called before any other
-   * class operations.  This can fail due to OOM conditions.
-   * @param initSize the initial number of buckets in the hashtable, default 16
-   * @return true if the class was initialized properly.
-   */
-  bool Init(PRUint32 initSize = PL_DHASH_MIN_SIZE);
-
-  /**
-   * Check whether the table has been initialized. This can be useful for static hashtables.
-   * @return the initialization state of the class.
-   */
-  bool IsInitialized() const { return !!mTable.entrySize; }
+  nsTHashtable(nsTHashtable<EntryType>&& aOther);
 
   /**
    * Return the generation number for the table. This increments whenever
    * the table data items are moved.
    */
-  PRUint32 GetGeneration() const { return mTable.generation; }
+  uint32_t GetGeneration() const { return mTable.generation; }
 
   /**
    * KeyType is typedef'ed for ease of use.
@@ -150,12 +121,12 @@ public:
    * Return the number of entries in the table.
    * @return    number of entries
    */
-  PRUint32 Count() const { return mTable.entryCount; }
+  uint32_t Count() const { return mTable.entryCount; }
 
   /**
    * Get the entry associated with a key.
    * @param     aKey the key to retrieve
-   * @return    pointer to the entry class, if the key exists; nsnull if the
+   * @return    pointer to the entry class, if the key exists; nullptr if the
    *            key doesn't exist
    */
   EntryType* GetEntry(KeyType aKey) const
@@ -168,19 +139,37 @@ public:
                             const_cast<PLDHashTable*>(&mTable),
                             EntryType::KeyToPointer(aKey),
                             PL_DHASH_LOOKUP));
-    return PL_DHASH_ENTRY_IS_BUSY(entry) ? entry : nsnull;
+    return PL_DHASH_ENTRY_IS_BUSY(entry) ? entry : nullptr;
+  }
+
+  /**
+   * Return true if an entry for the given key exists, false otherwise.
+   * @param     aKey the key to retrieve
+   * @return    true if the key exists, false if the key doesn't exist
+   */
+  bool Contains(KeyType aKey) const
+  {
+    return !!GetEntry(aKey);
   }
 
   /**
    * Get the entry associated with a key, or create a new entry,
    * @param     aKey the key to retrieve
-   * @return    pointer to the entry class retreived; nsnull only if memory
+   * @return    pointer to the entry class retreived; nullptr only if memory
                 can't be allocated
    */
   EntryType* PutEntry(KeyType aKey)
+  {
+    EntryType* e = PutEntry(aKey, fallible_t());
+    if (!e)
+      NS_ABORT_OOM(mTable.entrySize * mTable.entryCount);
+    return e;
+  }
+
+  EntryType* PutEntry(KeyType aKey, const fallible_t&) NS_WARN_UNUSED_RESULT
   {
     NS_ASSERTION(mTable.entrySize, "nsTHashtable was not initialized properly.");
-    
+
     return static_cast<EntryType*>
                       (PL_DHashTableOperate(
                             &mTable,
@@ -233,7 +222,7 @@ public:
    *            <code>Enumerator</code> function
    * @return    the number of entries actually enumerated
    */
-  PRUint32 EnumerateEntries(Enumerator enumFunc, void* userArg)
+  uint32_t EnumerateEntries(Enumerator enumFunc, void* userArg)
   {
     NS_ASSERTION(mTable.entrySize, "nsTHashtable was not initialized properly.");
     
@@ -248,17 +237,59 @@ public:
   {
     NS_ASSERTION(mTable.entrySize, "nsTHashtable was not initialized properly.");
 
-    PL_DHashTableEnumerate(&mTable, PL_DHashStubEnumRemove, nsnull);
+    PL_DHashTableEnumerate(&mTable, PL_DHashStubEnumRemove, nullptr);
   }
 
-  PRUint64 SizeOf()
+  /**
+   * client must provide a <code>SizeOfEntryExcludingThisFun</code> function for
+   *   SizeOfExcludingThis.
+   * @param     aEntry the entry being enumerated
+   * @param     mallocSizeOf the function used to measure heap-allocated blocks
+   * @param     arg passed unchanged from <code>SizeOf{In,Ex}cludingThis</code>
+   * @return    summed size of the things pointed to by the entries
+   */
+  typedef size_t (* SizeOfEntryExcludingThisFun)(EntryType* aEntry,
+                                                 mozilla::MallocSizeOf mallocSizeOf,
+                                                 void *arg);
+
+  /**
+   * Measure the size of the table's entry storage, and if
+   * |sizeOfEntryExcludingThis| is non-nullptr, measure the size of things
+   * pointed to by entries.
+   * 
+   * @param     sizeOfEntryExcludingThis the
+   *            <code>SizeOfEntryExcludingThisFun</code> function to call
+   * @param     mallocSizeOf the function used to measure heap-allocated blocks
+   * @param     userArg a pointer to pass to the
+   *            <code>SizeOfEntryExcludingThisFun</code> function
+   * @return    the summed size of all the entries
+   */
+  size_t SizeOfExcludingThis(SizeOfEntryExcludingThisFun sizeOfEntryExcludingThis,
+                             mozilla::MallocSizeOf mallocSizeOf,
+                             void *userArg = nullptr) const
   {
-    if (IsInitialized()) {
-      return PL_DHashTableSizeOf(&mTable);
+    if (sizeOfEntryExcludingThis) {
+      s_SizeOfArgs args = { sizeOfEntryExcludingThis, userArg };
+      return PL_DHashTableSizeOfExcludingThis(&mTable, s_SizeOfStub, mallocSizeOf, &args);
     }
-    return 0;
+    return PL_DHashTableSizeOfExcludingThis(&mTable, nullptr, mallocSizeOf);
   }
 
+#ifdef DEBUG
+  /**
+   * Mark the table as constant after initialization.
+   *
+   * This will prevent assertions when a read-only hash is accessed on multiple
+   * threads without synchronization.
+   */
+  void MarkImmutable()
+  {
+    NS_ASSERTION(mTable.entrySize, "nsTHashtable was not initialized properly.");
+
+    PL_DHashMarkTableImmutable(&mTable);
+  }
+#endif
+
 protected:
   PLDHashTable mTable;
 
@@ -298,14 +329,38 @@ protected:
   
   static PLDHashOperator s_EnumStub(PLDHashTable    *table,
                                     PLDHashEntryHdr *entry,
-                                    PRUint32         number,
+                                    uint32_t         number,
                                     void            *arg);
+
+  /**
+   * passed internally during sizeOf counting.  Allocated on the stack.
+   *
+   * @param userFunc the SizeOfEntryExcludingThisFun passed to
+   *                 SizeOf{In,Ex}cludingThis by the client
+   * @param userArg the userArg passed unaltered
+   */
+  struct s_SizeOfArgs
+  {
+    SizeOfEntryExcludingThisFun userFunc;
+    void* userArg;
+  };
+  
+  static size_t s_SizeOfStub(PLDHashEntryHdr *entry,
+                             mozilla::MallocSizeOf mallocSizeOf,
+                             void *arg);
+
 private:
   // copy constructor, not implemented
-  nsTHashtable(nsTHashtable<EntryType>& toCopy);
+  nsTHashtable(nsTHashtable<EntryType>& toCopy) MOZ_DELETE;
+
+  /**
+   * Initialize the table.
+   * @param initSize the initial number of buckets in the hashtable
+   */
+  void Init(uint32_t aInitSize);
 
   // assignment operator, not implemented
-  nsTHashtable<EntryType>& operator= (nsTHashtable<EntryType>& toEqual);
+  nsTHashtable<EntryType>& operator= (nsTHashtable<EntryType>& toEqual) MOZ_DELETE;
 };
 
 //
@@ -313,10 +368,17 @@ private:
 //
 
 template<class EntryType>
-nsTHashtable<EntryType>::nsTHashtable()
+nsTHashtable<EntryType>::nsTHashtable(
+  nsTHashtable<EntryType>&& aOther)
+  : mTable(mozilla::Move(aOther.mTable))
 {
-  // entrySize is our "I'm initialized" indicator
-  mTable.entrySize = 0;
+  // aOther shouldn't touch mTable after this, because we've stolen the table's
+  // pointers but not overwitten them.
+  MOZ_MAKE_MEM_UNDEFINED(&aOther.mTable, sizeof(aOther.mTable));
+
+  // Indicate that aOther is not initialized.  This will make its destructor a
+  // nop, which is what we want.
+  aOther.mTable.entrySize = 0;
 }
 
 template<class EntryType>
@@ -327,40 +389,22 @@ nsTHashtable<EntryType>::~nsTHashtable()
 }
 
 template<class EntryType>
-bool
-nsTHashtable<EntryType>::Init(PRUint32 initSize)
+void
+nsTHashtable<EntryType>::Init(uint32_t aInitSize)
 {
-  if (mTable.entrySize)
-  {
-    NS_ERROR("nsTHashtable::Init() should not be called twice.");
-    return true;
-  }
-
-  static PLDHashTableOps sOps = 
+  static const PLDHashTableOps sOps =
   {
     ::PL_DHashAllocTable,
     ::PL_DHashFreeTable,
     s_HashKey,
     s_MatchEntry,
-    ::PL_DHashMoveEntryStub,
+    EntryType::ALLOW_MEMMOVE ? ::PL_DHashMoveEntryStub : s_CopyEntry,
     s_ClearEntry,
     ::PL_DHashFinalizeStub,
     s_InitEntry
   };
 
-  if (!EntryType::ALLOW_MEMMOVE)
-  {
-    sOps.moveEntry = s_CopyEntry;
-  }
-  
-  if (!PL_DHashTableInit(&mTable, &sOps, nsnull, sizeof(EntryType), initSize))
-  {
-    // if failed, reset "flag"
-    mTable.entrySize = 0;
-    return false;
-  }
-
-  return true;
+  PL_DHashTableInit(&mTable, &sOps, nullptr, sizeof(EntryType), aInitSize);
 }
 
 // static definitions
@@ -392,7 +436,7 @@ nsTHashtable<EntryType>::s_CopyEntry(PLDHashTable          *table,
   EntryType* fromEntry =
     const_cast<EntryType*>(reinterpret_cast<const EntryType*>(from));
 
-  new(to) EntryType(*fromEntry);
+  new(to) EntryType(mozilla::Move(*fromEntry));
 
   fromEntry->~EntryType();
 }
@@ -419,7 +463,7 @@ template<class EntryType>
 PLDHashOperator
 nsTHashtable<EntryType>::s_EnumStub(PLDHashTable    *table,
                                     PLDHashEntryHdr *entry,
-                                    PRUint32         number,
+                                    uint32_t         number,
                                     void            *arg)
 {
   // dereferences the function-pointer to the user's enumeration function
@@ -428,4 +472,69 @@ nsTHashtable<EntryType>::s_EnumStub(PLDHashTable    *table,
     reinterpret_cast<s_EnumArgs*>(arg)->userArg);
 }
 
+template<class EntryType>
+size_t
+nsTHashtable<EntryType>::s_SizeOfStub(PLDHashEntryHdr *entry,
+                                      mozilla::MallocSizeOf mallocSizeOf,
+                                      void *arg)
+{
+  // dereferences the function-pointer to the user's enumeration function
+  return (* reinterpret_cast<s_SizeOfArgs*>(arg)->userFunc)(
+    reinterpret_cast<EntryType*>(entry),
+    mallocSizeOf,
+    reinterpret_cast<s_SizeOfArgs*>(arg)->userArg);
+}
+
+class nsCycleCollectionTraversalCallback;
+
+struct MOZ_STACK_CLASS nsTHashtableCCTraversalData
+{
+  nsTHashtableCCTraversalData(nsCycleCollectionTraversalCallback& aCallback,
+                              const char* aName,
+                              uint32_t aFlags)
+  : mCallback(aCallback),
+    mName(aName),
+    mFlags(aFlags)
+  {
+  }
+
+  nsCycleCollectionTraversalCallback& mCallback;
+  const char* mName;
+  uint32_t mFlags;
+};
+
+template <class EntryType>
+PLDHashOperator
+ImplCycleCollectionTraverse_EnumFunc(EntryType *aEntry,
+                                     void* aUserData)
+{
+  auto userData = static_cast<nsTHashtableCCTraversalData*>(aUserData);
+
+  ImplCycleCollectionTraverse(userData->mCallback,
+                              *aEntry,
+                              userData->mName,
+                              userData->mFlags);
+  return PL_DHASH_NEXT;
+}
+
+template <class EntryType>
+inline void
+ImplCycleCollectionUnlink(nsTHashtable<EntryType>& aField)
+{
+  aField.Clear();
+}
+
+template <class EntryType>
+inline void
+ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback,
+                            nsTHashtable<EntryType>& aField,
+                            const char* aName,
+                            uint32_t aFlags = 0)
+{
+  nsTHashtableCCTraversalData userData(aCallback, aName, aFlags);
+
+  aField.EnumerateEntries(ImplCycleCollectionTraverse_EnumFunc<EntryType>,
+                          &userData);
+}
+
 #endif // nsTHashtable_h__
